/********************************************************************************
** Form generated from reading UI file 'compliancereportdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COMPLIANCEREPORTDIALOG_H
#define UI_COMPLIANCEREPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ComplianceReportDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QLabel *label;
    QLineEdit *mEditProjectName;
    QLineEdit *mEditProjectVersion;
    QLabel *label_3;
    QComboBox *mCodingStandard;
    QCheckBox *mCheckFiles;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ComplianceReportDialog)
    {
        if (ComplianceReportDialog->objectName().isEmpty())
            ComplianceReportDialog->setObjectName("ComplianceReportDialog");
        ComplianceReportDialog->resize(403, 199);
        verticalLayout = new QVBoxLayout(ComplianceReportDialog);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(ComplianceReportDialog);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        label = new QLabel(ComplianceReportDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        mEditProjectName = new QLineEdit(ComplianceReportDialog);
        mEditProjectName->setObjectName("mEditProjectName");

        gridLayout->addWidget(mEditProjectName, 1, 1, 1, 1);

        mEditProjectVersion = new QLineEdit(ComplianceReportDialog);
        mEditProjectVersion->setObjectName("mEditProjectVersion");

        gridLayout->addWidget(mEditProjectVersion, 2, 1, 1, 1);

        label_3 = new QLabel(ComplianceReportDialog);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        mCodingStandard = new QComboBox(ComplianceReportDialog);
        mCodingStandard->addItem(QString());
        mCodingStandard->addItem(QString());
        mCodingStandard->addItem(QString());
        mCodingStandard->setObjectName("mCodingStandard");

        gridLayout->addWidget(mCodingStandard, 0, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        mCheckFiles = new QCheckBox(ComplianceReportDialog);
        mCheckFiles->setObjectName("mCheckFiles");
        mCheckFiles->setChecked(true);

        verticalLayout->addWidget(mCheckFiles);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(ComplianceReportDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close|QDialogButtonBox::Save);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ComplianceReportDialog);

        QMetaObject::connectSlotsByName(ComplianceReportDialog);
    } // setupUi

    void retranslateUi(QDialog *ComplianceReportDialog)
    {
        ComplianceReportDialog->setWindowTitle(QCoreApplication::translate("ComplianceReportDialog", "Compliance Report", nullptr));
        label_2->setText(QCoreApplication::translate("ComplianceReportDialog", "Project version", nullptr));
        label->setText(QCoreApplication::translate("ComplianceReportDialog", "Project name", nullptr));
        label_3->setText(QCoreApplication::translate("ComplianceReportDialog", "Coding Standard", nullptr));
        mCodingStandard->setItemText(0, QCoreApplication::translate("ComplianceReportDialog", "Misra C", nullptr));
        mCodingStandard->setItemText(1, QCoreApplication::translate("ComplianceReportDialog", "Cert C", nullptr));
        mCodingStandard->setItemText(2, QCoreApplication::translate("ComplianceReportDialog", "Cert C++", nullptr));

        mCheckFiles->setText(QCoreApplication::translate("ComplianceReportDialog", "List of files with md5 checksums", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ComplianceReportDialog: public Ui_ComplianceReportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COMPLIANCEREPORTDIALOG_H
