/********************************************************************************
** Form generated from reading UI file 'librarydialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LIBRARYDIALOG_H
#define UI_LIBRARYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LibraryDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *buttonOpen;
    QPushButton *buttonSave;
    QPushButton *buttonSaveAs;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *sortFunctions;
    QSpacerItem *horizontalSpacer_4;
    QListWidget *functions;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *addFunction;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_4;
    QLineEdit *filter;
    QVBoxLayout *verticalLayout_2;
    QWidget *widget;
    QHBoxLayout *horizontalLayout_7;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_5;
    QSpacerItem *verticalSpacer;
    QPlainTextEdit *comments;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label;
    QComboBox *noreturn;
    QCheckBox *useretval;
    QCheckBox *leakignore;
    QLabel *label_3;
    QListWidget *arguments;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *editArgButton;
    QSpacerItem *horizontalSpacer_3;

    void setupUi(QDialog *LibraryDialog)
    {
        if (LibraryDialog->objectName().isEmpty())
            LibraryDialog->setObjectName("LibraryDialog");
        LibraryDialog->resize(869, 588);
        verticalLayout_3 = new QVBoxLayout(LibraryDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        buttonOpen = new QPushButton(LibraryDialog);
        buttonOpen->setObjectName("buttonOpen");

        horizontalLayout_2->addWidget(buttonOpen);

        buttonSave = new QPushButton(LibraryDialog);
        buttonSave->setObjectName("buttonSave");

        horizontalLayout_2->addWidget(buttonSave);

        buttonSaveAs = new QPushButton(LibraryDialog);
        buttonSaveAs->setObjectName("buttonSaveAs");

        horizontalLayout_2->addWidget(buttonSaveAs);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label_2 = new QLabel(LibraryDialog);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        sortFunctions = new QPushButton(LibraryDialog);
        sortFunctions->setObjectName("sortFunctions");
        sortFunctions->setCheckable(true);
        sortFunctions->setChecked(false);

        horizontalLayout_6->addWidget(sortFunctions);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_4);


        verticalLayout->addLayout(horizontalLayout_6);

        functions = new QListWidget(LibraryDialog);
        functions->setObjectName("functions");
        functions->setEditTriggers(QAbstractItemView::DoubleClicked);
        functions->setSelectionBehavior(QAbstractItemView::SelectRows);

        verticalLayout->addWidget(functions);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        addFunction = new QPushButton(LibraryDialog);
        addFunction->setObjectName("addFunction");

        horizontalLayout_3->addWidget(addFunction);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        label_4 = new QLabel(LibraryDialog);
        label_4->setObjectName("label_4");

        horizontalLayout_3->addWidget(label_4);

        filter = new QLineEdit(LibraryDialog);
        filter->setObjectName("filter");

        horizontalLayout_3->addWidget(filter);


        verticalLayout->addLayout(horizontalLayout_3);


        horizontalLayout->addLayout(verticalLayout);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        widget = new QWidget(LibraryDialog);
        widget->setObjectName("widget");
        widget->setMinimumSize(QSize(0, 32));
        widget->setMaximumSize(QSize(16777215, 80));
        horizontalLayout_7 = new QHBoxLayout(widget);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_5 = new QLabel(widget);
        label_5->setObjectName("label_5");

        verticalLayout_4->addWidget(label_5);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        horizontalLayout_7->addLayout(verticalLayout_4);

        comments = new QPlainTextEdit(widget);
        comments->setObjectName("comments");
        comments->setLineWrapMode(QPlainTextEdit::NoWrap);

        horizontalLayout_7->addWidget(comments);


        verticalLayout_2->addWidget(widget);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label = new QLabel(LibraryDialog);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        horizontalLayout_5->addWidget(label);

        noreturn = new QComboBox(LibraryDialog);
        noreturn->addItem(QString());
        noreturn->addItem(QString());
        noreturn->addItem(QString());
        noreturn->setObjectName("noreturn");

        horizontalLayout_5->addWidget(noreturn);


        verticalLayout_2->addLayout(horizontalLayout_5);

        useretval = new QCheckBox(LibraryDialog);
        useretval->setObjectName("useretval");

        verticalLayout_2->addWidget(useretval);

        leakignore = new QCheckBox(LibraryDialog);
        leakignore->setObjectName("leakignore");

        verticalLayout_2->addWidget(leakignore);

        label_3 = new QLabel(LibraryDialog);
        label_3->setObjectName("label_3");

        verticalLayout_2->addWidget(label_3);

        arguments = new QListWidget(LibraryDialog);
        arguments->setObjectName("arguments");

        verticalLayout_2->addWidget(arguments);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        editArgButton = new QPushButton(LibraryDialog);
        editArgButton->setObjectName("editArgButton");

        horizontalLayout_4->addWidget(editArgButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        verticalLayout_2->addLayout(horizontalLayout_4);


        horizontalLayout->addLayout(verticalLayout_2);


        verticalLayout_3->addLayout(horizontalLayout);


        retranslateUi(LibraryDialog);
        QObject::connect(buttonOpen, SIGNAL(clicked()), LibraryDialog, SLOT(openCfg()));
        QObject::connect(useretval, SIGNAL(clicked()), LibraryDialog, SLOT(changeFunction()));
        QObject::connect(leakignore, SIGNAL(clicked()), LibraryDialog, SLOT(changeFunction()));
        QObject::connect(buttonSave, SIGNAL(clicked()), LibraryDialog, SLOT(saveCfg()));
        QObject::connect(addFunction, SIGNAL(clicked()), LibraryDialog, SLOT(addFunction()));
        QObject::connect(editArgButton, SIGNAL(clicked()), LibraryDialog, SLOT(editArg()));
        QObject::connect(arguments, SIGNAL(itemDoubleClicked(QListWidgetItem*)), LibraryDialog, SLOT(editArg()));
        QObject::connect(sortFunctions, SIGNAL(toggled(bool)), LibraryDialog, SLOT(sortFunctions(bool)));
        QObject::connect(filter, SIGNAL(textChanged(QString)), LibraryDialog, SLOT(filterFunctions(QString)));
        QObject::connect(functions, SIGNAL(itemSelectionChanged()), LibraryDialog, SLOT(selectFunction()));
        QObject::connect(noreturn, SIGNAL(currentIndexChanged(int)), LibraryDialog, SLOT(changeFunction()));
        QObject::connect(functions, SIGNAL(itemChanged(QListWidgetItem*)), LibraryDialog, SLOT(editFunctionName(QListWidgetItem*)));
        QObject::connect(noreturn, SIGNAL(editTextChanged(QString)), LibraryDialog, SLOT(changeFunction()));
        QObject::connect(comments, SIGNAL(textChanged()), LibraryDialog, SLOT(changeFunction()));
        QObject::connect(buttonSaveAs, SIGNAL(clicked()), LibraryDialog, SLOT(saveCfgAs()));

        QMetaObject::connectSlotsByName(LibraryDialog);
    } // setupUi

    void retranslateUi(QDialog *LibraryDialog)
    {
        LibraryDialog->setWindowTitle(QCoreApplication::translate("LibraryDialog", "Library Editor", nullptr));
        buttonOpen->setText(QCoreApplication::translate("LibraryDialog", "Open", nullptr));
        buttonSave->setText(QCoreApplication::translate("LibraryDialog", "Save", nullptr));
        buttonSaveAs->setText(QCoreApplication::translate("LibraryDialog", "Save as", nullptr));
        label_2->setText(QCoreApplication::translate("LibraryDialog", "Functions", nullptr));
        sortFunctions->setText(QCoreApplication::translate("LibraryDialog", "Sort", nullptr));
        addFunction->setText(QCoreApplication::translate("LibraryDialog", "Add", nullptr));
        label_4->setText(QCoreApplication::translate("LibraryDialog", "Filter:", nullptr));
        label_5->setText(QCoreApplication::translate("LibraryDialog", "Comments", nullptr));
        label->setText(QCoreApplication::translate("LibraryDialog", "noreturn", nullptr));
        noreturn->setItemText(0, QCoreApplication::translate("LibraryDialog", "False", nullptr));
        noreturn->setItemText(1, QCoreApplication::translate("LibraryDialog", "True", nullptr));
        noreturn->setItemText(2, QCoreApplication::translate("LibraryDialog", "Unknown", nullptr));

        useretval->setText(QCoreApplication::translate("LibraryDialog", "return value must be used", nullptr));
        leakignore->setText(QCoreApplication::translate("LibraryDialog", "ignore function in leaks checking", nullptr));
        label_3->setText(QCoreApplication::translate("LibraryDialog", "Arguments", nullptr));
        editArgButton->setText(QCoreApplication::translate("LibraryDialog", "Edit", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LibraryDialog: public Ui_LibraryDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LIBRARYDIALOG_H
