/********************************************************************************
** Form generated from reading UI file 'statsdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STATSDIALOG_H
#define UI_STATSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_StatsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *mTabWidget;
    QWidget *mProjectTab;
    QVBoxLayout *verticalLayout_2;
    QLabel *mProjectLabel;
    QLineEdit *mProject;
    QLabel *mPathsLabel;
    QLineEdit *mPaths;
    QLabel *mIncludePathsLabel;
    QLineEdit *mIncludePaths;
    QLabel *mDefinesLabel;
    QLineEdit *mDefines;
    QLabel *mUndefinesLabel;
    QLineEdit *mUndefines;
    QSpacerItem *mVerticalSpacer;
    QWidget *mScanTab;
    QGridLayout *gridLayout;
    QLabel *mPathLabel;
    QLabel *mNumberOfFilesScannedLabel;
    QLabel *mNumberOfFilesScanned;
    QLabel *mScanDurationLabel;
    QLabel *mScanDuration;
    QSpacerItem *mVerticalSpacer_2;
    QLineEdit *mPath;
    QWidget *mStatsTab;
    QGridLayout *gridLayout_2;
    QLabel *label;
    QLabel *mLblErrors;
    QLabel *label_2;
    QLabel *mLblWarnings;
    QLabel *label_3;
    QLabel *mLblStyle;
    QLabel *label_6;
    QLabel *mLblPortability;
    QLabel *label_4;
    QLabel *mLblPerformance;
    QLabel *label_5;
    QLabel *mLblInformation;
    QLabel *label_7;
    QLabel *mLblActiveCheckers;
    QWidget *mCheckersTab;
    QVBoxLayout *verticalLayout_4;
    QPlainTextEdit *mCheckersReport;
    QWidget *mTabHistory;
    QVBoxLayout *verticalLayout_3;
    QLabel *mLblHistoryFile;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *mCopyToClipboard;
    QPushButton *mPDFexport;
    QDialogButtonBox *mButtonBox;

    void setupUi(QDialog *StatsDialog)
    {
        if (StatsDialog->objectName().isEmpty())
            StatsDialog->setObjectName("StatsDialog");
        StatsDialog->resize(500, 414);
        verticalLayout = new QVBoxLayout(StatsDialog);
        verticalLayout->setObjectName("verticalLayout");
        mTabWidget = new QTabWidget(StatsDialog);
        mTabWidget->setObjectName("mTabWidget");
        mTabWidget->setTabShape(QTabWidget::Rounded);
        mProjectTab = new QWidget();
        mProjectTab->setObjectName("mProjectTab");
        verticalLayout_2 = new QVBoxLayout(mProjectTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mProjectLabel = new QLabel(mProjectTab);
        mProjectLabel->setObjectName("mProjectLabel");

        verticalLayout_2->addWidget(mProjectLabel);

        mProject = new QLineEdit(mProjectTab);
        mProject->setObjectName("mProject");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mProject->sizePolicy().hasHeightForWidth());
        mProject->setSizePolicy(sizePolicy);
        mProject->setReadOnly(true);

        verticalLayout_2->addWidget(mProject);

        mPathsLabel = new QLabel(mProjectTab);
        mPathsLabel->setObjectName("mPathsLabel");

        verticalLayout_2->addWidget(mPathsLabel);

        mPaths = new QLineEdit(mProjectTab);
        mPaths->setObjectName("mPaths");
        mPaths->setEnabled(true);
        sizePolicy.setHeightForWidth(mPaths->sizePolicy().hasHeightForWidth());
        mPaths->setSizePolicy(sizePolicy);
        mPaths->setAcceptDrops(true);
        mPaths->setAutoFillBackground(false);
        mPaths->setFrame(true);
        mPaths->setReadOnly(true);

        verticalLayout_2->addWidget(mPaths);

        mIncludePathsLabel = new QLabel(mProjectTab);
        mIncludePathsLabel->setObjectName("mIncludePathsLabel");

        verticalLayout_2->addWidget(mIncludePathsLabel);

        mIncludePaths = new QLineEdit(mProjectTab);
        mIncludePaths->setObjectName("mIncludePaths");
        mIncludePaths->setEnabled(true);
        sizePolicy.setHeightForWidth(mIncludePaths->sizePolicy().hasHeightForWidth());
        mIncludePaths->setSizePolicy(sizePolicy);
        mIncludePaths->setReadOnly(true);

        verticalLayout_2->addWidget(mIncludePaths);

        mDefinesLabel = new QLabel(mProjectTab);
        mDefinesLabel->setObjectName("mDefinesLabel");

        verticalLayout_2->addWidget(mDefinesLabel);

        mDefines = new QLineEdit(mProjectTab);
        mDefines->setObjectName("mDefines");
        mDefines->setEnabled(true);
        sizePolicy.setHeightForWidth(mDefines->sizePolicy().hasHeightForWidth());
        mDefines->setSizePolicy(sizePolicy);
        mDefines->setReadOnly(true);

        verticalLayout_2->addWidget(mDefines);

        mUndefinesLabel = new QLabel(mProjectTab);
        mUndefinesLabel->setObjectName("mUndefinesLabel");

        verticalLayout_2->addWidget(mUndefinesLabel);

        mUndefines = new QLineEdit(mProjectTab);
        mUndefines->setObjectName("mUndefines");
        sizePolicy.setHeightForWidth(mUndefines->sizePolicy().hasHeightForWidth());
        mUndefines->setSizePolicy(sizePolicy);
        mUndefines->setReadOnly(true);

        verticalLayout_2->addWidget(mUndefines);

        mVerticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(mVerticalSpacer);

        mTabWidget->addTab(mProjectTab, QString());
        mScanTab = new QWidget();
        mScanTab->setObjectName("mScanTab");
        gridLayout = new QGridLayout(mScanTab);
        gridLayout->setObjectName("gridLayout");
        mPathLabel = new QLabel(mScanTab);
        mPathLabel->setObjectName("mPathLabel");
        mPathLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(mPathLabel, 0, 0, 1, 1);

        mNumberOfFilesScannedLabel = new QLabel(mScanTab);
        mNumberOfFilesScannedLabel->setObjectName("mNumberOfFilesScannedLabel");
        mNumberOfFilesScannedLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(mNumberOfFilesScannedLabel, 1, 0, 1, 1);

        mNumberOfFilesScanned = new QLabel(mScanTab);
        mNumberOfFilesScanned->setObjectName("mNumberOfFilesScanned");
        mNumberOfFilesScanned->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(mNumberOfFilesScanned, 1, 1, 1, 1);

        mScanDurationLabel = new QLabel(mScanTab);
        mScanDurationLabel->setObjectName("mScanDurationLabel");
        mScanDurationLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(mScanDurationLabel, 2, 0, 1, 1);

        mScanDuration = new QLabel(mScanTab);
        mScanDuration->setObjectName("mScanDuration");
        mScanDuration->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(mScanDuration, 2, 1, 1, 1);

        mVerticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(mVerticalSpacer_2, 3, 0, 1, 1);

        mPath = new QLineEdit(mScanTab);
        mPath->setObjectName("mPath");
        sizePolicy.setHeightForWidth(mPath->sizePolicy().hasHeightForWidth());
        mPath->setSizePolicy(sizePolicy);
        mPath->setReadOnly(true);

        gridLayout->addWidget(mPath, 0, 1, 1, 1);

        mTabWidget->addTab(mScanTab, QString());
        mStatsTab = new QWidget();
        mStatsTab->setObjectName("mStatsTab");
        gridLayout_2 = new QGridLayout(mStatsTab);
        gridLayout_2->setObjectName("gridLayout_2");
        label = new QLabel(mStatsTab);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        mLblErrors = new QLabel(mStatsTab);
        mLblErrors->setObjectName("mLblErrors");
        mLblErrors->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblErrors, 0, 1, 1, 1);

        label_2 = new QLabel(mStatsTab);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        mLblWarnings = new QLabel(mStatsTab);
        mLblWarnings->setObjectName("mLblWarnings");
        mLblWarnings->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblWarnings, 1, 1, 1, 1);

        label_3 = new QLabel(mStatsTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_3, 2, 0, 1, 1);

        mLblStyle = new QLabel(mStatsTab);
        mLblStyle->setObjectName("mLblStyle");
        mLblStyle->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblStyle, 2, 1, 1, 1);

        label_6 = new QLabel(mStatsTab);
        label_6->setObjectName("label_6");
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_6, 3, 0, 1, 1);

        mLblPortability = new QLabel(mStatsTab);
        mLblPortability->setObjectName("mLblPortability");
        mLblPortability->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblPortability, 3, 1, 1, 1);

        label_4 = new QLabel(mStatsTab);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_4, 4, 0, 1, 1);

        mLblPerformance = new QLabel(mStatsTab);
        mLblPerformance->setObjectName("mLblPerformance");
        mLblPerformance->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblPerformance, 4, 1, 1, 1);

        label_5 = new QLabel(mStatsTab);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_5, 5, 0, 1, 1);

        mLblInformation = new QLabel(mStatsTab);
        mLblInformation->setObjectName("mLblInformation");
        mLblInformation->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblInformation, 5, 1, 1, 1);

        label_7 = new QLabel(mStatsTab);
        label_7->setObjectName("label_7");
        label_7->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_7, 6, 0, 1, 1);

        mLblActiveCheckers = new QLabel(mStatsTab);
        mLblActiveCheckers->setObjectName("mLblActiveCheckers");
        mLblActiveCheckers->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(mLblActiveCheckers, 6, 1, 1, 1);

        mTabWidget->addTab(mStatsTab, QString());
        mCheckersTab = new QWidget();
        mCheckersTab->setObjectName("mCheckersTab");
        verticalLayout_4 = new QVBoxLayout(mCheckersTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        mCheckersReport = new QPlainTextEdit(mCheckersTab);
        mCheckersReport->setObjectName("mCheckersReport");
        mCheckersReport->setEnabled(true);
        QFont font;
        font.setFamilies({QString::fromUtf8("Courier 10 Pitch")});
        mCheckersReport->setFont(font);
        mCheckersReport->setUndoRedoEnabled(true);
        mCheckersReport->setReadOnly(true);

        verticalLayout_4->addWidget(mCheckersReport);

        mTabWidget->addTab(mCheckersTab, QString());
        mTabHistory = new QWidget();
        mTabHistory->setObjectName("mTabHistory");
        verticalLayout_3 = new QVBoxLayout(mTabHistory);
        verticalLayout_3->setObjectName("verticalLayout_3");
        mLblHistoryFile = new QLabel(mTabHistory);
        mLblHistoryFile->setObjectName("mLblHistoryFile");

        verticalLayout_3->addWidget(mLblHistoryFile);

        mTabWidget->addTab(mTabHistory, QString());

        verticalLayout->addWidget(mTabWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        mCopyToClipboard = new QPushButton(StatsDialog);
        mCopyToClipboard->setObjectName("mCopyToClipboard");

        horizontalLayout->addWidget(mCopyToClipboard);

        mPDFexport = new QPushButton(StatsDialog);
        mPDFexport->setObjectName("mPDFexport");

        horizontalLayout->addWidget(mPDFexport);

        mButtonBox = new QDialogButtonBox(StatsDialog);
        mButtonBox->setObjectName("mButtonBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mButtonBox->sizePolicy().hasHeightForWidth());
        mButtonBox->setSizePolicy(sizePolicy1);
        mButtonBox->setOrientation(Qt::Horizontal);
        mButtonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(mButtonBox);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(mTabWidget, mProject);
        QWidget::setTabOrder(mProject, mPaths);
        QWidget::setTabOrder(mPaths, mIncludePaths);
        QWidget::setTabOrder(mIncludePaths, mDefines);
        QWidget::setTabOrder(mDefines, mCopyToClipboard);
        QWidget::setTabOrder(mCopyToClipboard, mButtonBox);
        QWidget::setTabOrder(mButtonBox, mPath);

        retranslateUi(StatsDialog);
        QObject::connect(mButtonBox, &QDialogButtonBox::accepted, StatsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(mButtonBox, &QDialogButtonBox::rejected, StatsDialog, qOverload<>(&QDialog::reject));

        mTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(StatsDialog);
    } // setupUi

    void retranslateUi(QDialog *StatsDialog)
    {
        StatsDialog->setWindowTitle(QCoreApplication::translate("StatsDialog", "Statistics", nullptr));
        mProjectLabel->setText(QCoreApplication::translate("StatsDialog", "Project:", nullptr));
        mPathsLabel->setText(QCoreApplication::translate("StatsDialog", "Paths:", nullptr));
        mIncludePathsLabel->setText(QCoreApplication::translate("StatsDialog", "Include paths:", nullptr));
        mDefinesLabel->setText(QCoreApplication::translate("StatsDialog", "Defines:", nullptr));
        mUndefinesLabel->setText(QCoreApplication::translate("StatsDialog", "Undefines:", nullptr));
        mTabWidget->setTabText(mTabWidget->indexOf(mProjectTab), QCoreApplication::translate("StatsDialog", "Project", nullptr));
        mPathLabel->setText(QCoreApplication::translate("StatsDialog", "Path Selected:", nullptr));
        mNumberOfFilesScannedLabel->setText(QCoreApplication::translate("StatsDialog", "Number of Files Scanned:", nullptr));
        mNumberOfFilesScanned->setText(QString());
        mScanDurationLabel->setText(QCoreApplication::translate("StatsDialog", "Scan Duration:", nullptr));
        mScanDuration->setText(QString());
        mTabWidget->setTabText(mTabWidget->indexOf(mScanTab), QCoreApplication::translate("StatsDialog", "Previous Scan", nullptr));
        label->setText(QCoreApplication::translate("StatsDialog", "Errors:", nullptr));
        label_2->setText(QCoreApplication::translate("StatsDialog", "Warnings:", nullptr));
        label_3->setText(QCoreApplication::translate("StatsDialog", "Stylistic warnings:", nullptr));
        label_6->setText(QCoreApplication::translate("StatsDialog", "Portability warnings:", nullptr));
        label_4->setText(QCoreApplication::translate("StatsDialog", "Performance issues:", nullptr));
        label_5->setText(QCoreApplication::translate("StatsDialog", "Information messages:", nullptr));
        label_7->setText(QCoreApplication::translate("StatsDialog", "Active checkers:", nullptr));
        mTabWidget->setTabText(mTabWidget->indexOf(mStatsTab), QCoreApplication::translate("StatsDialog", "Statistics", nullptr));
        mTabWidget->setTabText(mTabWidget->indexOf(mCheckersTab), QCoreApplication::translate("StatsDialog", "Checkers", nullptr));
        mLblHistoryFile->setText(QCoreApplication::translate("StatsDialog", "File:", nullptr));
        mTabWidget->setTabText(mTabWidget->indexOf(mTabHistory), QCoreApplication::translate("StatsDialog", "History", nullptr));
        mCopyToClipboard->setText(QCoreApplication::translate("StatsDialog", "Copy to Clipboard", nullptr));
        mPDFexport->setText(QCoreApplication::translate("StatsDialog", "Pdf Export", nullptr));
    } // retranslateUi

};

namespace Ui {
    class StatsDialog: public Ui_StatsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STATSDIALOG_H
