/********************************************************************************
** Form generated from reading UI file 'projectfile.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROJECTFILE_H
#define UI_PROJECTFILE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ProjectFile
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *mTabPathsAndDefines;
    QVBoxLayout *verticalLayout_8;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_17;
    QHBoxLayout *horizontalLayout;
    QLineEdit *mEditImportProject;
    QPushButton *mBtnClearImportProject;
    QPushButton *mBtnBrowseImportProject;
    QHBoxLayout *horizontalLayout_8;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *mChkAllVsConfigs;
    QSpacerItem *verticalSpacer_11;
    QSpacerItem *horizontalSpacer_2;
    QVBoxLayout *verticalLayout_16;
    QLabel *mLabelVSConfig;
    QSpacerItem *verticalSpacer_12;
    QListWidget *mListVsConfigs;
    QHBoxLayout *mLayoutCheckPaths;
    QVBoxLayout *verticalLayout_5;
    QLabel *mLabelCheckPaths;
    QSpacerItem *verticalSpacer_2;
    QListWidget *mListCheckPaths;
    QVBoxLayout *mLayoutCheckPathsButtons;
    QPushButton *mBtnAddCheckPath;
    QPushButton *mBtnEditCheckPath;
    QPushButton *mBtnRemoveCheckPath;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *mLayoutDefines;
    QLabel *mLabelDefines;
    QLineEdit *mEditDefines;
    QHBoxLayout *mLayoutUndefines;
    QLabel *mLabelUndefines;
    QLineEdit *mEditUndefines;
    QHBoxLayout *mLayoutIncludePaths;
    QVBoxLayout *verticalLayout_11;
    QLabel *mLabelIncludePaths;
    QSpacerItem *verticalSpacer_6;
    QListWidget *mListIncludeDirs;
    QVBoxLayout *mLayoutIncludePathsButtons;
    QPushButton *mBtnAddInclude;
    QPushButton *mBtnEditInclude;
    QPushButton *mBtnRemoveInclude;
    QSpacerItem *verticalSpacer;
    QPushButton *mBtnIncludeUp;
    QPushButton *mBtnIncludeDown;
    QSpacerItem *verticalSpacer_8;
    QWidget *mTabTypesAndFunctions;
    QVBoxLayout *verticalLayout_15;
    QGroupBox *groupBox_6;
    QVBoxLayout *verticalLayout_12;
    QComboBox *mComboBoxPlatform;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QListWidget *mLibraries;
    QLabel *mLabelLibrariesNote;
    QWidget *mTabAnalysis;
    QVBoxLayout *verticalLayout_22;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *mEditBuildDir;
    QPushButton *mBtnBrowseBuildDir;
    QGroupBox *groupBox_10;
    QVBoxLayout *verticalLayout_14;
    QRadioButton *mBtnCppcheckParser;
    QRadioButton *mBtnClangParser;
    QGroupBox *groupBox_13;
    QVBoxLayout *verticalLayout_18;
    QRadioButton *mCheckLevelReduced;
    QRadioButton *mCheckLevelNormal;
    QRadioButton *mCheckLevelExhaustive;
    QGroupBox *groupBox_11;
    QVBoxLayout *verticalLayout_19;
    QCheckBox *mBtnSafeClasses;
    QGroupBox *groupBox_9;
    QVBoxLayout *verticalLayout_9;
    QCheckBox *mCheckHeaders;
    QCheckBox *mCheckUnusedTemplates;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_3;
    QSpinBox *mMaxCtuDepth;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_4;
    QSpinBox *mMaxTemplateRecursion;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *premiumLicense;
    QHBoxLayout *horizontalLayout_12;
    QLineEdit *mEditLicenseFile;
    QPushButton *mBtnBrowseLicense;
    QWidget *mTabWarningOptions;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_13;
    QLineEdit *mEditProjectRoot;
    QGroupBox *groupBox_5;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *mEditTags;
    QGroupBox *groupBox_7;
    QHBoxLayout *horizontalLayout_4;
    QListWidget *mListExcludedPaths;
    QVBoxLayout *layoutExcludePathButtons;
    QPushButton *mBtnAddIgnorePath;
    QPushButton *mBtnAddIgnoreFile;
    QPushButton *mBtnEditIgnorePath;
    QPushButton *mBtnRemoveIgnorePath;
    QSpacerItem *verticalSpacer_4;
    QGroupBox *groupBox_8;
    QHBoxLayout *horizontalLayout_5;
    QListWidget *mListSuppressions;
    QVBoxLayout *verticalLayout_6;
    QPushButton *mBtnAddSuppression;
    QPushButton *mBtnRemoveSuppression;
    QSpacerItem *verticalSpacer_10;
    QCheckBox *mInlineSuppressions;
    QSpacerItem *verticalSpacer_9;
    QWidget *mTabAddons;
    QVBoxLayout *verticalLayout_10;
    QGroupBox *mGroupBoxAddons;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_addons_require_python;
    QCheckBox *mAddonY2038;
    QCheckBox *mAddonThreadSafety;
    QGroupBox *groupBox_12;
    QVBoxLayout *verticalLayout_20;
    QHBoxLayout *horizontalLayout_6;
    QCheckBox *mMisraC;
    QComboBox *mMisraVersion;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *mLayoutMisraRuleTexts;
    QLabel *mLabelMisraFile;
    QLineEdit *mEditMisraFile;
    QPushButton *mBtnBrowseMisraFile;
    QHBoxLayout *horizontalLayout_11;
    QCheckBox *mMisraCpp;
    QComboBox *mMisraCppVersion;
    QSpacerItem *horizontalSpacer_5;
    QCheckBox *mCertC2016;
    QHBoxLayout *horizontalLayout_10;
    QLabel *mLabelCertIntPrecision;
    QLineEdit *mEditCertIntPrecision;
    QCheckBox *mCertCpp2016;
    QCheckBox *mAutosar;
    QCheckBox *mSafetyProfiles;
    QGroupBox *mGroupboxBughunting;
    QVBoxLayout *verticalLayout_21;
    QCheckBox *mBughunting;
    QGroupBox *mGroupBoxTools;
    QVBoxLayout *verticalLayout_1;
    QCheckBox *mToolClangTidy;
    QCheckBox *mToolClangAnalyzer;
    QSpacerItem *verticalSpacer_5;
    QDialogButtonBox *mButtons;

    void setupUi(QDialog *ProjectFile)
    {
        if (ProjectFile->objectName().isEmpty())
            ProjectFile->setObjectName("ProjectFile");
        ProjectFile->resize(940, 701);
        verticalLayout = new QVBoxLayout(ProjectFile);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(ProjectFile);
        tabWidget->setObjectName("tabWidget");
        mTabPathsAndDefines = new QWidget();
        mTabPathsAndDefines->setObjectName("mTabPathsAndDefines");
        verticalLayout_8 = new QVBoxLayout(mTabPathsAndDefines);
        verticalLayout_8->setObjectName("verticalLayout_8");
        groupBox_4 = new QGroupBox(mTabPathsAndDefines);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout_17 = new QVBoxLayout(groupBox_4);
        verticalLayout_17->setObjectName("verticalLayout_17");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mEditImportProject = new QLineEdit(groupBox_4);
        mEditImportProject->setObjectName("mEditImportProject");
        mEditImportProject->setReadOnly(true);

        horizontalLayout->addWidget(mEditImportProject);

        mBtnClearImportProject = new QPushButton(groupBox_4);
        mBtnClearImportProject->setObjectName("mBtnClearImportProject");
        mBtnClearImportProject->setEnabled(false);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/edit-clear.png"), QSize(), QIcon::Mode::Active, QIcon::State::On);
        mBtnClearImportProject->setIcon(icon);

        horizontalLayout->addWidget(mBtnClearImportProject);

        mBtnBrowseImportProject = new QPushButton(groupBox_4);
        mBtnBrowseImportProject->setObjectName("mBtnBrowseImportProject");

        horizontalLayout->addWidget(mBtnBrowseImportProject);


        verticalLayout_17->addLayout(horizontalLayout);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        mChkAllVsConfigs = new QCheckBox(groupBox_4);
        mChkAllVsConfigs->setObjectName("mChkAllVsConfigs");

        verticalLayout_4->addWidget(mChkAllVsConfigs);

        verticalSpacer_11 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_11);


        horizontalLayout_8->addLayout(verticalLayout_4);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_2);

        verticalLayout_16 = new QVBoxLayout();
        verticalLayout_16->setObjectName("verticalLayout_16");
        mLabelVSConfig = new QLabel(groupBox_4);
        mLabelVSConfig->setObjectName("mLabelVSConfig");

        verticalLayout_16->addWidget(mLabelVSConfig);

        verticalSpacer_12 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_16->addItem(verticalSpacer_12);


        horizontalLayout_8->addLayout(verticalLayout_16);

        mListVsConfigs = new QListWidget(groupBox_4);
        mListVsConfigs->setObjectName("mListVsConfigs");

        horizontalLayout_8->addWidget(mListVsConfigs);


        verticalLayout_17->addLayout(horizontalLayout_8);


        verticalLayout_8->addWidget(groupBox_4);

        mLayoutCheckPaths = new QHBoxLayout();
        mLayoutCheckPaths->setObjectName("mLayoutCheckPaths");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        mLabelCheckPaths = new QLabel(mTabPathsAndDefines);
        mLabelCheckPaths->setObjectName("mLabelCheckPaths");

        verticalLayout_5->addWidget(mLabelCheckPaths);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_2);


        mLayoutCheckPaths->addLayout(verticalLayout_5);

        mListCheckPaths = new QListWidget(mTabPathsAndDefines);
        mListCheckPaths->setObjectName("mListCheckPaths");
        mListCheckPaths->setMaximumSize(QSize(16777215, 140));

        mLayoutCheckPaths->addWidget(mListCheckPaths);

        mLayoutCheckPathsButtons = new QVBoxLayout();
        mLayoutCheckPathsButtons->setObjectName("mLayoutCheckPathsButtons");
        mBtnAddCheckPath = new QPushButton(mTabPathsAndDefines);
        mBtnAddCheckPath->setObjectName("mBtnAddCheckPath");

        mLayoutCheckPathsButtons->addWidget(mBtnAddCheckPath);

        mBtnEditCheckPath = new QPushButton(mTabPathsAndDefines);
        mBtnEditCheckPath->setObjectName("mBtnEditCheckPath");

        mLayoutCheckPathsButtons->addWidget(mBtnEditCheckPath);

        mBtnRemoveCheckPath = new QPushButton(mTabPathsAndDefines);
        mBtnRemoveCheckPath->setObjectName("mBtnRemoveCheckPath");

        mLayoutCheckPathsButtons->addWidget(mBtnRemoveCheckPath);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mLayoutCheckPathsButtons->addItem(verticalSpacer_3);


        mLayoutCheckPaths->addLayout(mLayoutCheckPathsButtons);


        verticalLayout_8->addLayout(mLayoutCheckPaths);

        mLayoutDefines = new QHBoxLayout();
        mLayoutDefines->setObjectName("mLayoutDefines");
        mLabelDefines = new QLabel(mTabPathsAndDefines);
        mLabelDefines->setObjectName("mLabelDefines");

        mLayoutDefines->addWidget(mLabelDefines);

        mEditDefines = new QLineEdit(mTabPathsAndDefines);
        mEditDefines->setObjectName("mEditDefines");

        mLayoutDefines->addWidget(mEditDefines);


        verticalLayout_8->addLayout(mLayoutDefines);

        mLayoutUndefines = new QHBoxLayout();
        mLayoutUndefines->setObjectName("mLayoutUndefines");
        mLabelUndefines = new QLabel(mTabPathsAndDefines);
        mLabelUndefines->setObjectName("mLabelUndefines");

        mLayoutUndefines->addWidget(mLabelUndefines);

        mEditUndefines = new QLineEdit(mTabPathsAndDefines);
        mEditUndefines->setObjectName("mEditUndefines");

        mLayoutUndefines->addWidget(mEditUndefines);


        verticalLayout_8->addLayout(mLayoutUndefines);

        mLayoutIncludePaths = new QHBoxLayout();
        mLayoutIncludePaths->setObjectName("mLayoutIncludePaths");
        verticalLayout_11 = new QVBoxLayout();
        verticalLayout_11->setObjectName("verticalLayout_11");
        mLabelIncludePaths = new QLabel(mTabPathsAndDefines);
        mLabelIncludePaths->setObjectName("mLabelIncludePaths");

        verticalLayout_11->addWidget(mLabelIncludePaths);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_11->addItem(verticalSpacer_6);


        mLayoutIncludePaths->addLayout(verticalLayout_11);

        mListIncludeDirs = new QListWidget(mTabPathsAndDefines);
        mListIncludeDirs->setObjectName("mListIncludeDirs");
        mListIncludeDirs->setSelectionBehavior(QAbstractItemView::SelectRows);

        mLayoutIncludePaths->addWidget(mListIncludeDirs);

        mLayoutIncludePathsButtons = new QVBoxLayout();
        mLayoutIncludePathsButtons->setObjectName("mLayoutIncludePathsButtons");
        mBtnAddInclude = new QPushButton(mTabPathsAndDefines);
        mBtnAddInclude->setObjectName("mBtnAddInclude");

        mLayoutIncludePathsButtons->addWidget(mBtnAddInclude);

        mBtnEditInclude = new QPushButton(mTabPathsAndDefines);
        mBtnEditInclude->setObjectName("mBtnEditInclude");

        mLayoutIncludePathsButtons->addWidget(mBtnEditInclude);

        mBtnRemoveInclude = new QPushButton(mTabPathsAndDefines);
        mBtnRemoveInclude->setObjectName("mBtnRemoveInclude");

        mLayoutIncludePathsButtons->addWidget(mBtnRemoveInclude);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mLayoutIncludePathsButtons->addItem(verticalSpacer);

        mBtnIncludeUp = new QPushButton(mTabPathsAndDefines);
        mBtnIncludeUp->setObjectName("mBtnIncludeUp");

        mLayoutIncludePathsButtons->addWidget(mBtnIncludeUp);

        mBtnIncludeDown = new QPushButton(mTabPathsAndDefines);
        mBtnIncludeDown->setObjectName("mBtnIncludeDown");

        mLayoutIncludePathsButtons->addWidget(mBtnIncludeDown);


        mLayoutIncludePaths->addLayout(mLayoutIncludePathsButtons);


        verticalLayout_8->addLayout(mLayoutIncludePaths);

        verticalSpacer_8 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_8);

        tabWidget->addTab(mTabPathsAndDefines, QString());
        mTabTypesAndFunctions = new QWidget();
        mTabTypesAndFunctions->setObjectName("mTabTypesAndFunctions");
        verticalLayout_15 = new QVBoxLayout(mTabTypesAndFunctions);
        verticalLayout_15->setObjectName("verticalLayout_15");
        groupBox_6 = new QGroupBox(mTabTypesAndFunctions);
        groupBox_6->setObjectName("groupBox_6");
        verticalLayout_12 = new QVBoxLayout(groupBox_6);
        verticalLayout_12->setObjectName("verticalLayout_12");
        mComboBoxPlatform = new QComboBox(groupBox_6);
        mComboBoxPlatform->setObjectName("mComboBoxPlatform");

        verticalLayout_12->addWidget(mComboBoxPlatform);


        verticalLayout_15->addWidget(groupBox_6);

        groupBox_3 = new QGroupBox(mTabTypesAndFunctions);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mLibraries = new QListWidget(groupBox_3);
        mLibraries->setObjectName("mLibraries");

        verticalLayout_2->addWidget(mLibraries);

        mLabelLibrariesNote = new QLabel(groupBox_3);
        mLabelLibrariesNote->setObjectName("mLabelLibrariesNote");
        mLabelLibrariesNote->setWordWrap(true);

        verticalLayout_2->addWidget(mLabelLibrariesNote);


        verticalLayout_15->addWidget(groupBox_3);

        tabWidget->addTab(mTabTypesAndFunctions, QString());
        mTabAnalysis = new QWidget();
        mTabAnalysis->setObjectName("mTabAnalysis");
        verticalLayout_22 = new QVBoxLayout(mTabAnalysis);
        verticalLayout_22->setObjectName("verticalLayout_22");
        groupBox_2 = new QGroupBox(mTabAnalysis);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_2 = new QHBoxLayout(groupBox_2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        mEditBuildDir = new QLineEdit(groupBox_2);
        mEditBuildDir->setObjectName("mEditBuildDir");

        horizontalLayout_2->addWidget(mEditBuildDir);

        mBtnBrowseBuildDir = new QPushButton(groupBox_2);
        mBtnBrowseBuildDir->setObjectName("mBtnBrowseBuildDir");

        horizontalLayout_2->addWidget(mBtnBrowseBuildDir);


        verticalLayout_22->addWidget(groupBox_2);

        groupBox_10 = new QGroupBox(mTabAnalysis);
        groupBox_10->setObjectName("groupBox_10");
        verticalLayout_14 = new QVBoxLayout(groupBox_10);
        verticalLayout_14->setObjectName("verticalLayout_14");
        mBtnCppcheckParser = new QRadioButton(groupBox_10);
        mBtnCppcheckParser->setObjectName("mBtnCppcheckParser");
        mBtnCppcheckParser->setChecked(true);

        verticalLayout_14->addWidget(mBtnCppcheckParser);

        mBtnClangParser = new QRadioButton(groupBox_10);
        mBtnClangParser->setObjectName("mBtnClangParser");

        verticalLayout_14->addWidget(mBtnClangParser);


        verticalLayout_22->addWidget(groupBox_10);

        groupBox_13 = new QGroupBox(mTabAnalysis);
        groupBox_13->setObjectName("groupBox_13");
        verticalLayout_18 = new QVBoxLayout(groupBox_13);
        verticalLayout_18->setObjectName("verticalLayout_18");
        mCheckLevelReduced = new QRadioButton(groupBox_13);
        mCheckLevelReduced->setObjectName("mCheckLevelReduced");

        verticalLayout_18->addWidget(mCheckLevelReduced);

        mCheckLevelNormal = new QRadioButton(groupBox_13);
        mCheckLevelNormal->setObjectName("mCheckLevelNormal");

        verticalLayout_18->addWidget(mCheckLevelNormal);

        mCheckLevelExhaustive = new QRadioButton(groupBox_13);
        mCheckLevelExhaustive->setObjectName("mCheckLevelExhaustive");

        verticalLayout_18->addWidget(mCheckLevelExhaustive);


        verticalLayout_22->addWidget(groupBox_13);

        groupBox_11 = new QGroupBox(mTabAnalysis);
        groupBox_11->setObjectName("groupBox_11");
        verticalLayout_19 = new QVBoxLayout(groupBox_11);
        verticalLayout_19->setObjectName("verticalLayout_19");
        mBtnSafeClasses = new QCheckBox(groupBox_11);
        mBtnSafeClasses->setObjectName("mBtnSafeClasses");

        verticalLayout_19->addWidget(mBtnSafeClasses);


        verticalLayout_22->addWidget(groupBox_11);

        groupBox_9 = new QGroupBox(mTabAnalysis);
        groupBox_9->setObjectName("groupBox_9");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox_9->sizePolicy().hasHeightForWidth());
        groupBox_9->setSizePolicy(sizePolicy);
        verticalLayout_9 = new QVBoxLayout(groupBox_9);
        verticalLayout_9->setObjectName("verticalLayout_9");
        mCheckHeaders = new QCheckBox(groupBox_9);
        mCheckHeaders->setObjectName("mCheckHeaders");
        mCheckHeaders->setChecked(true);

        verticalLayout_9->addWidget(mCheckHeaders);

        mCheckUnusedTemplates = new QCheckBox(groupBox_9);
        mCheckUnusedTemplates->setObjectName("mCheckUnusedTemplates");
        mCheckUnusedTemplates->setChecked(true);

        verticalLayout_9->addWidget(mCheckUnusedTemplates);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label_3 = new QLabel(groupBox_9);
        label_3->setObjectName("label_3");

        horizontalLayout_7->addWidget(label_3);

        mMaxCtuDepth = new QSpinBox(groupBox_9);
        mMaxCtuDepth->setObjectName("mMaxCtuDepth");
        mMaxCtuDepth->setValue(10);

        horizontalLayout_7->addWidget(mMaxCtuDepth);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);


        verticalLayout_9->addLayout(horizontalLayout_7);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        label_4 = new QLabel(groupBox_9);
        label_4->setObjectName("label_4");

        horizontalLayout_9->addWidget(label_4);

        mMaxTemplateRecursion = new QSpinBox(groupBox_9);
        mMaxTemplateRecursion->setObjectName("mMaxTemplateRecursion");
        mMaxTemplateRecursion->setMaximum(1000);
        mMaxTemplateRecursion->setValue(100);

        horizontalLayout_9->addWidget(mMaxTemplateRecursion);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_3);


        verticalLayout_9->addLayout(horizontalLayout_9);


        verticalLayout_22->addWidget(groupBox_9);

        premiumLicense = new QGroupBox(mTabAnalysis);
        premiumLicense->setObjectName("premiumLicense");
        horizontalLayout_12 = new QHBoxLayout(premiumLicense);
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        mEditLicenseFile = new QLineEdit(premiumLicense);
        mEditLicenseFile->setObjectName("mEditLicenseFile");

        horizontalLayout_12->addWidget(mEditLicenseFile);

        mBtnBrowseLicense = new QPushButton(premiumLicense);
        mBtnBrowseLicense->setObjectName("mBtnBrowseLicense");

        horizontalLayout_12->addWidget(mBtnBrowseLicense);


        verticalLayout_22->addWidget(premiumLicense);

        tabWidget->addTab(mTabAnalysis, QString());
        mTabWarningOptions = new QWidget();
        mTabWarningOptions->setObjectName("mTabWarningOptions");
        verticalLayout_7 = new QVBoxLayout(mTabWarningOptions);
        verticalLayout_7->setObjectName("verticalLayout_7");
        groupBox = new QGroupBox(mTabWarningOptions);
        groupBox->setObjectName("groupBox");
        verticalLayout_13 = new QVBoxLayout(groupBox);
        verticalLayout_13->setObjectName("verticalLayout_13");
        mEditProjectRoot = new QLineEdit(groupBox);
        mEditProjectRoot->setObjectName("mEditProjectRoot");

        verticalLayout_13->addWidget(mEditProjectRoot);


        verticalLayout_7->addWidget(groupBox);

        groupBox_5 = new QGroupBox(mTabWarningOptions);
        groupBox_5->setObjectName("groupBox_5");
        horizontalLayout_3 = new QHBoxLayout(groupBox_5);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        mEditTags = new QLineEdit(groupBox_5);
        mEditTags->setObjectName("mEditTags");

        horizontalLayout_3->addWidget(mEditTags);


        verticalLayout_7->addWidget(groupBox_5);

        groupBox_7 = new QGroupBox(mTabWarningOptions);
        groupBox_7->setObjectName("groupBox_7");
        horizontalLayout_4 = new QHBoxLayout(groupBox_7);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        mListExcludedPaths = new QListWidget(groupBox_7);
        mListExcludedPaths->setObjectName("mListExcludedPaths");

        horizontalLayout_4->addWidget(mListExcludedPaths);

        layoutExcludePathButtons = new QVBoxLayout();
        layoutExcludePathButtons->setObjectName("layoutExcludePathButtons");
        mBtnAddIgnorePath = new QPushButton(groupBox_7);
        mBtnAddIgnorePath->setObjectName("mBtnAddIgnorePath");

        layoutExcludePathButtons->addWidget(mBtnAddIgnorePath);

        mBtnAddIgnoreFile = new QPushButton(groupBox_7);
        mBtnAddIgnoreFile->setObjectName("mBtnAddIgnoreFile");

        layoutExcludePathButtons->addWidget(mBtnAddIgnoreFile);

        mBtnEditIgnorePath = new QPushButton(groupBox_7);
        mBtnEditIgnorePath->setObjectName("mBtnEditIgnorePath");

        layoutExcludePathButtons->addWidget(mBtnEditIgnorePath);

        mBtnRemoveIgnorePath = new QPushButton(groupBox_7);
        mBtnRemoveIgnorePath->setObjectName("mBtnRemoveIgnorePath");

        layoutExcludePathButtons->addWidget(mBtnRemoveIgnorePath);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        layoutExcludePathButtons->addItem(verticalSpacer_4);


        horizontalLayout_4->addLayout(layoutExcludePathButtons);


        verticalLayout_7->addWidget(groupBox_7);

        groupBox_8 = new QGroupBox(mTabWarningOptions);
        groupBox_8->setObjectName("groupBox_8");
        horizontalLayout_5 = new QHBoxLayout(groupBox_8);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        mListSuppressions = new QListWidget(groupBox_8);
        mListSuppressions->setObjectName("mListSuppressions");

        horizontalLayout_5->addWidget(mListSuppressions);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        mBtnAddSuppression = new QPushButton(groupBox_8);
        mBtnAddSuppression->setObjectName("mBtnAddSuppression");

        verticalLayout_6->addWidget(mBtnAddSuppression);

        mBtnRemoveSuppression = new QPushButton(groupBox_8);
        mBtnRemoveSuppression->setObjectName("mBtnRemoveSuppression");

        verticalLayout_6->addWidget(mBtnRemoveSuppression);

        verticalSpacer_10 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_10);


        horizontalLayout_5->addLayout(verticalLayout_6);


        verticalLayout_7->addWidget(groupBox_8);

        mInlineSuppressions = new QCheckBox(mTabWarningOptions);
        mInlineSuppressions->setObjectName("mInlineSuppressions");

        verticalLayout_7->addWidget(mInlineSuppressions);

        verticalSpacer_9 = new QSpacerItem(20, 96, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_9);

        tabWidget->addTab(mTabWarningOptions, QString());
        mTabAddons = new QWidget();
        mTabAddons->setObjectName("mTabAddons");
        verticalLayout_10 = new QVBoxLayout(mTabAddons);
        verticalLayout_10->setObjectName("verticalLayout_10");
        mGroupBoxAddons = new QGroupBox(mTabAddons);
        mGroupBoxAddons->setObjectName("mGroupBoxAddons");
        verticalLayout_3 = new QVBoxLayout(mGroupBoxAddons);
        verticalLayout_3->setObjectName("verticalLayout_3");
        label_addons_require_python = new QLabel(mGroupBoxAddons);
        label_addons_require_python->setObjectName("label_addons_require_python");
        label_addons_require_python->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(label_addons_require_python);

        mAddonY2038 = new QCheckBox(mGroupBoxAddons);
        mAddonY2038->setObjectName("mAddonY2038");

        verticalLayout_3->addWidget(mAddonY2038);

        mAddonThreadSafety = new QCheckBox(mGroupBoxAddons);
        mAddonThreadSafety->setObjectName("mAddonThreadSafety");

        verticalLayout_3->addWidget(mAddonThreadSafety);


        verticalLayout_10->addWidget(mGroupBoxAddons);

        groupBox_12 = new QGroupBox(mTabAddons);
        groupBox_12->setObjectName("groupBox_12");
        verticalLayout_20 = new QVBoxLayout(groupBox_12);
        verticalLayout_20->setObjectName("verticalLayout_20");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        mMisraC = new QCheckBox(groupBox_12);
        mMisraC->setObjectName("mMisraC");

        horizontalLayout_6->addWidget(mMisraC);

        mMisraVersion = new QComboBox(groupBox_12);
        mMisraVersion->addItem(QString());
        mMisraVersion->addItem(QString());
        mMisraVersion->addItem(QString());
        mMisraVersion->setObjectName("mMisraVersion");

        horizontalLayout_6->addWidget(mMisraVersion);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_4);


        verticalLayout_20->addLayout(horizontalLayout_6);

        mLayoutMisraRuleTexts = new QHBoxLayout();
        mLayoutMisraRuleTexts->setObjectName("mLayoutMisraRuleTexts");
        mLabelMisraFile = new QLabel(groupBox_12);
        mLabelMisraFile->setObjectName("mLabelMisraFile");

        mLayoutMisraRuleTexts->addWidget(mLabelMisraFile);

        mEditMisraFile = new QLineEdit(groupBox_12);
        mEditMisraFile->setObjectName("mEditMisraFile");

        mLayoutMisraRuleTexts->addWidget(mEditMisraFile);

        mBtnBrowseMisraFile = new QPushButton(groupBox_12);
        mBtnBrowseMisraFile->setObjectName("mBtnBrowseMisraFile");

        mLayoutMisraRuleTexts->addWidget(mBtnBrowseMisraFile);


        verticalLayout_20->addLayout(mLayoutMisraRuleTexts);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        mMisraCpp = new QCheckBox(groupBox_12);
        mMisraCpp->setObjectName("mMisraCpp");

        horizontalLayout_11->addWidget(mMisraCpp);

        mMisraCppVersion = new QComboBox(groupBox_12);
        mMisraCppVersion->addItem(QString());
        mMisraCppVersion->addItem(QString());
        mMisraCppVersion->setObjectName("mMisraCppVersion");

        horizontalLayout_11->addWidget(mMisraCppVersion);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_5);


        verticalLayout_20->addLayout(horizontalLayout_11);

        mCertC2016 = new QCheckBox(groupBox_12);
        mCertC2016->setObjectName("mCertC2016");

        verticalLayout_20->addWidget(mCertC2016);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        mLabelCertIntPrecision = new QLabel(groupBox_12);
        mLabelCertIntPrecision->setObjectName("mLabelCertIntPrecision");

        horizontalLayout_10->addWidget(mLabelCertIntPrecision);

        mEditCertIntPrecision = new QLineEdit(groupBox_12);
        mEditCertIntPrecision->setObjectName("mEditCertIntPrecision");

        horizontalLayout_10->addWidget(mEditCertIntPrecision);


        verticalLayout_20->addLayout(horizontalLayout_10);

        mCertCpp2016 = new QCheckBox(groupBox_12);
        mCertCpp2016->setObjectName("mCertCpp2016");

        verticalLayout_20->addWidget(mCertCpp2016);

        mAutosar = new QCheckBox(groupBox_12);
        mAutosar->setObjectName("mAutosar");

        verticalLayout_20->addWidget(mAutosar);

        mSafetyProfiles = new QCheckBox(groupBox_12);
        mSafetyProfiles->setObjectName("mSafetyProfiles");

        verticalLayout_20->addWidget(mSafetyProfiles);


        verticalLayout_10->addWidget(groupBox_12);

        mGroupboxBughunting = new QGroupBox(mTabAddons);
        mGroupboxBughunting->setObjectName("mGroupboxBughunting");
        verticalLayout_21 = new QVBoxLayout(mGroupboxBughunting);
        verticalLayout_21->setObjectName("verticalLayout_21");
        mBughunting = new QCheckBox(mGroupboxBughunting);
        mBughunting->setObjectName("mBughunting");

        verticalLayout_21->addWidget(mBughunting);


        verticalLayout_10->addWidget(mGroupboxBughunting);

        mGroupBoxTools = new QGroupBox(mTabAddons);
        mGroupBoxTools->setObjectName("mGroupBoxTools");
        verticalLayout_1 = new QVBoxLayout(mGroupBoxTools);
        verticalLayout_1->setObjectName("verticalLayout_1");
        mToolClangTidy = new QCheckBox(mGroupBoxTools);
        mToolClangTidy->setObjectName("mToolClangTidy");

        verticalLayout_1->addWidget(mToolClangTidy);

        mToolClangAnalyzer = new QCheckBox(mGroupBoxTools);
        mToolClangAnalyzer->setObjectName("mToolClangAnalyzer");

        verticalLayout_1->addWidget(mToolClangAnalyzer);


        verticalLayout_10->addWidget(mGroupBoxTools);

        verticalSpacer_5 = new QSpacerItem(20, 368, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_5);

        tabWidget->addTab(mTabAddons, QString());

        verticalLayout->addWidget(tabWidget);

        mButtons = new QDialogButtonBox(ProjectFile);
        mButtons->setObjectName("mButtons");
        mButtons->setOrientation(Qt::Horizontal);
        mButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(mButtons);

#if QT_CONFIG(shortcut)
        mLabelDefines->setBuddy(mEditDefines);
        mLabelUndefines->setBuddy(mEditUndefines);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ProjectFile);
        QObject::connect(mButtons, &QDialogButtonBox::accepted, ProjectFile, qOverload<>(&QDialog::accept));
        QObject::connect(mButtons, &QDialogButtonBox::rejected, ProjectFile, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ProjectFile);
    } // setupUi

    void retranslateUi(QDialog *ProjectFile)
    {
        ProjectFile->setWindowTitle(QCoreApplication::translate("ProjectFile", "Project File", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("ProjectFile", "Import Project (Visual studio / compile database/ Borland C++ Builder 6)", nullptr));
        mBtnClearImportProject->setText(QString());
        mBtnBrowseImportProject->setText(QCoreApplication::translate("ProjectFile", "Browse...", nullptr));
#if QT_CONFIG(tooltip)
        mChkAllVsConfigs->setToolTip(QCoreApplication::translate("ProjectFile", "<html><head/><body><p>You have a choice:</p><p> * Analyze all Debug and Release configurations</p><p> * Only analyze the first matching Debug configuration</p><p><br/></p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        mChkAllVsConfigs->setText(QCoreApplication::translate("ProjectFile", "Analyze all Visual Studio configurations", nullptr));
        mLabelVSConfig->setText(QCoreApplication::translate("ProjectFile", "Selected VS Configurations", nullptr));
        mLabelCheckPaths->setText(QCoreApplication::translate("ProjectFile", "Paths:", nullptr));
        mBtnAddCheckPath->setText(QCoreApplication::translate("ProjectFile", "Add...", nullptr));
        mBtnEditCheckPath->setText(QCoreApplication::translate("ProjectFile", "Edit", nullptr));
        mBtnRemoveCheckPath->setText(QCoreApplication::translate("ProjectFile", "Remove", nullptr));
        mLabelDefines->setText(QCoreApplication::translate("ProjectFile", "Defines:", nullptr));
#if QT_CONFIG(tooltip)
        mEditDefines->setToolTip(QCoreApplication::translate("ProjectFile", "Defines must be separated by a semicolon. Example: DEF1;DEF2=5;DEF3=int", nullptr));
#endif // QT_CONFIG(tooltip)
        mLabelUndefines->setText(QCoreApplication::translate("ProjectFile", "Undefines:", nullptr));
#if QT_CONFIG(tooltip)
        mEditUndefines->setToolTip(QCoreApplication::translate("ProjectFile", "Undefines must be separated by a semicolon. Example: UNDEF1;UNDEF2;UNDEF3", nullptr));
#endif // QT_CONFIG(tooltip)
        mLabelIncludePaths->setText(QCoreApplication::translate("ProjectFile", "Include Paths:", nullptr));
        mBtnAddInclude->setText(QCoreApplication::translate("ProjectFile", "Add...", nullptr));
        mBtnEditInclude->setText(QCoreApplication::translate("ProjectFile", "Edit", nullptr));
        mBtnRemoveInclude->setText(QCoreApplication::translate("ProjectFile", "Remove", nullptr));
        mBtnIncludeUp->setText(QCoreApplication::translate("ProjectFile", "Up", nullptr));
        mBtnIncludeDown->setText(QCoreApplication::translate("ProjectFile", "Down", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabPathsAndDefines), QCoreApplication::translate("ProjectFile", "Paths and Defines", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("ProjectFile", "Platform", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("ProjectFile", "Libraries", nullptr));
        mLabelLibrariesNote->setText(QCoreApplication::translate("ProjectFile", "Note: Put your own custom .cfg files in the same folder as the project file. You should see them above.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabTypesAndFunctions), QCoreApplication::translate("ProjectFile", "Types and Functions", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ProjectFile", "Cppcheck build dir (whole program analysis, incremental analysis, statistics, etc)", nullptr));
#if QT_CONFIG(tooltip)
        mEditBuildDir->setToolTip(QCoreApplication::translate("ProjectFile", "This is a workfolder that Cppcheck will use for various purposes.", nullptr));
#endif // QT_CONFIG(tooltip)
        mBtnBrowseBuildDir->setText(QCoreApplication::translate("ProjectFile", "Browse...", nullptr));
        groupBox_10->setTitle(QCoreApplication::translate("ProjectFile", "Parser", nullptr));
        mBtnCppcheckParser->setText(QCoreApplication::translate("ProjectFile", "Cppcheck (built in)", nullptr));
        mBtnClangParser->setText(QCoreApplication::translate("ProjectFile", "Clang (experimental)", nullptr));
        groupBox_13->setTitle(QCoreApplication::translate("ProjectFile", "Check level", nullptr));
        mCheckLevelReduced->setText(QCoreApplication::translate("ProjectFile", "Reduced -- meant for usage where developer wants results directly. Limited and faster analysis with fewer results.", nullptr));
        mCheckLevelNormal->setText(QCoreApplication::translate("ProjectFile", "Normal -- meant for normal analysis in CI. Analysis should finish in reasonable time.", nullptr));
        mCheckLevelExhaustive->setText(QCoreApplication::translate("ProjectFile", "Exhaustive -- meant for nightly builds etc. Analysis time can be longer (10x slower than compilation is OK).", nullptr));
        groupBox_11->setTitle(QCoreApplication::translate("ProjectFile", "Analysis", nullptr));
#if QT_CONFIG(tooltip)
        mBtnSafeClasses->setToolTip(QCoreApplication::translate("ProjectFile", "If you want to design your classes to be as flexible and robust as possible then the public interface must be very robust. Cppcheck will asumme that arguments can take *any* value.", nullptr));
#endif // QT_CONFIG(tooltip)
        mBtnSafeClasses->setText(QCoreApplication::translate("ProjectFile", "Check that each class has a safe public interface", nullptr));
        groupBox_9->setTitle(QCoreApplication::translate("ProjectFile", "Limit analysis", nullptr));
        mCheckHeaders->setText(QCoreApplication::translate("ProjectFile", "Check code in headers  (should be ON normally. if you want a limited quick analysis then turn this OFF)", nullptr));
        mCheckUnusedTemplates->setText(QCoreApplication::translate("ProjectFile", "Check code in unused templates (should be ON normally, however in theory you can safely ignore warnings in unused templates)", nullptr));
        label_3->setText(QCoreApplication::translate("ProjectFile", "Max CTU depth", nullptr));
        label_4->setText(QCoreApplication::translate("ProjectFile", "Max recursion in template instantiation", nullptr));
        premiumLicense->setTitle(QCoreApplication::translate("ProjectFile", "Premium License", nullptr));
        mBtnBrowseLicense->setText(QCoreApplication::translate("ProjectFile", "Browse...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabAnalysis), QCoreApplication::translate("ProjectFile", "Analysis", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ProjectFile", "Root path:", nullptr));
#if QT_CONFIG(tooltip)
        mEditProjectRoot->setToolTip(QCoreApplication::translate("ProjectFile", "Filepaths in warnings will be relative to this path", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_5->setTitle(QCoreApplication::translate("ProjectFile", "Warning tags (separated by semicolon)", nullptr));
#if QT_CONFIG(tooltip)
        mEditTags->setToolTip(QCoreApplication::translate("ProjectFile", "If tags are added, you will be able to right click on warnings and set one of these tags. You can manually categorize warnings.", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_7->setTitle(QCoreApplication::translate("ProjectFile", "Exclude source files", nullptr));
        mBtnAddIgnorePath->setText(QCoreApplication::translate("ProjectFile", "Exclude folder...", nullptr));
        mBtnAddIgnoreFile->setText(QCoreApplication::translate("ProjectFile", "Exclude file...", nullptr));
        mBtnEditIgnorePath->setText(QCoreApplication::translate("ProjectFile", "Edit", nullptr));
        mBtnRemoveIgnorePath->setText(QCoreApplication::translate("ProjectFile", "Remove", nullptr));
        groupBox_8->setTitle(QCoreApplication::translate("ProjectFile", "Suppressions", nullptr));
        mBtnAddSuppression->setText(QCoreApplication::translate("ProjectFile", "Add", nullptr));
        mBtnRemoveSuppression->setText(QCoreApplication::translate("ProjectFile", "Remove", nullptr));
        mInlineSuppressions->setText(QCoreApplication::translate("ProjectFile", "Enable inline suppressions", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabWarningOptions), QCoreApplication::translate("ProjectFile", "Warning options", nullptr));
        mGroupBoxAddons->setTitle(QCoreApplication::translate("ProjectFile", "Addons", nullptr));
        label_addons_require_python->setText(QCoreApplication::translate("ProjectFile", "Note: Addons require <a href=\"https://www.python.org/\">Python</a> being installed.", nullptr));
        mAddonY2038->setText(QCoreApplication::translate("ProjectFile", "Y2038", nullptr));
        mAddonThreadSafety->setText(QCoreApplication::translate("ProjectFile", "Thread safety", nullptr));
        groupBox_12->setTitle(QCoreApplication::translate("ProjectFile", "Coding standards", nullptr));
        mMisraC->setText(QCoreApplication::translate("ProjectFile", "Misra C", nullptr));
        mMisraVersion->setItemText(0, QCoreApplication::translate("ProjectFile", "2012", nullptr));
        mMisraVersion->setItemText(1, QCoreApplication::translate("ProjectFile", "2023", nullptr));
        mMisraVersion->setItemText(2, QCoreApplication::translate("ProjectFile", "2025", nullptr));

        mLabelMisraFile->setText(QCoreApplication::translate("ProjectFile", "MISRA rule texts", nullptr));
#if QT_CONFIG(tooltip)
        mEditMisraFile->setToolTip(QCoreApplication::translate("ProjectFile", "<html><head/><body><p>Copy/paste the text from Appendix A &quot;Summary of guidelines&quot; from the MISRA C 2012 pdf to a text file.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        mBtnBrowseMisraFile->setText(QCoreApplication::translate("ProjectFile", "...", nullptr));
        mMisraCpp->setText(QCoreApplication::translate("ProjectFile", "Misra C++", nullptr));
        mMisraCppVersion->setItemText(0, QCoreApplication::translate("ProjectFile", "2008", nullptr));
        mMisraCppVersion->setItemText(1, QCoreApplication::translate("ProjectFile", "2023", nullptr));

        mCertC2016->setText(QCoreApplication::translate("ProjectFile", "Cert C", nullptr));
        mLabelCertIntPrecision->setText(QCoreApplication::translate("ProjectFile", "CERT-INT35-C:  int precision (if size equals precision, you can leave empty)", nullptr));
        mCertCpp2016->setText(QCoreApplication::translate("ProjectFile", "Cert C++", nullptr));
        mAutosar->setText(QCoreApplication::translate("ProjectFile", "Autosar", nullptr));
        mSafetyProfiles->setText(QCoreApplication::translate("ProjectFile", "Safety profiles (defined in C++ core guidelines)", nullptr));
        mGroupboxBughunting->setTitle(QCoreApplication::translate("ProjectFile", "Bug hunting (Premium)", nullptr));
        mBughunting->setText(QCoreApplication::translate("ProjectFile", "Bug hunting", nullptr));
        mGroupBoxTools->setTitle(QCoreApplication::translate("ProjectFile", "External tools", nullptr));
        mToolClangTidy->setText(QCoreApplication::translate("ProjectFile", "Clang-tidy", nullptr));
        mToolClangAnalyzer->setText(QCoreApplication::translate("ProjectFile", "Clang analyzer", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabAddons), QCoreApplication::translate("ProjectFile", "Addons", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ProjectFile: public Ui_ProjectFile {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROJECTFILE_H
