/********************************************************************************
** Form generated from reading UI file 'libraryaddfunctiondialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LIBRARYADDFUNCTIONDIALOG_H
#define UI_LIBRARYADDFUNCTIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LibraryAddFunctionDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    QLineEdit *functionName;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout;
    QSpinBox *numberOfArguments;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LibraryAddFunctionDialog)
    {
        if (LibraryAddFunctionDialog->objectName().isEmpty())
            LibraryAddFunctionDialog->setObjectName("LibraryAddFunctionDialog");
        LibraryAddFunctionDialog->setWindowModality(Qt::WindowModal);
        LibraryAddFunctionDialog->resize(353, 89);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LibraryAddFunctionDialog->sizePolicy().hasHeightForWidth());
        LibraryAddFunctionDialog->setSizePolicy(sizePolicy);
        LibraryAddFunctionDialog->setModal(true);
        verticalLayout = new QVBoxLayout(LibraryAddFunctionDialog);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(LibraryAddFunctionDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        functionName = new QLineEdit(LibraryAddFunctionDialog);
        functionName->setObjectName("functionName");

        gridLayout->addWidget(functionName, 0, 1, 1, 1);

        label_2 = new QLabel(LibraryAddFunctionDialog);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        numberOfArguments = new QSpinBox(LibraryAddFunctionDialog);
        numberOfArguments->setObjectName("numberOfArguments");

        horizontalLayout->addWidget(numberOfArguments);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        buttonBox = new QDialogButtonBox(LibraryAddFunctionDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(LibraryAddFunctionDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, LibraryAddFunctionDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, LibraryAddFunctionDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(LibraryAddFunctionDialog);
    } // setupUi

    void retranslateUi(QDialog *LibraryAddFunctionDialog)
    {
        LibraryAddFunctionDialog->setWindowTitle(QCoreApplication::translate("LibraryAddFunctionDialog", "Add function", nullptr));
        label->setText(QCoreApplication::translate("LibraryAddFunctionDialog", "Function name(s)", nullptr));
        label_2->setText(QCoreApplication::translate("LibraryAddFunctionDialog", "Number of arguments", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LibraryAddFunctionDialog: public Ui_LibraryAddFunctionDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LIBRARYADDFUNCTIONDIALOG_H
