/********************************************************************************
** Form generated from reading UI file 'helpdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HELPDIALOG_H
#define UI_HELPDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "helpdialog.h"

QT_BEGIN_NAMESPACE

class Ui_HelpDialog
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    QTabWidget *tabs;
    QWidget *tabContents;
    QVBoxLayout *verticalLayout_5;
    QVBoxLayout *contents;
    QWidget *tabIndex;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *index;
    HelpBrowser *textBrowser;

    void setupUi(QDialog *HelpDialog)
    {
        if (HelpDialog->objectName().isEmpty())
            HelpDialog->setObjectName("HelpDialog");
        HelpDialog->resize(635, 446);
        verticalLayout = new QVBoxLayout(HelpDialog);
        verticalLayout->setObjectName("verticalLayout");
        splitter = new QSplitter(HelpDialog);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        tabs = new QTabWidget(splitter);
        tabs->setObjectName("tabs");
        tabs->setMaximumSize(QSize(200, 16777215));
        tabContents = new QWidget();
        tabContents->setObjectName("tabContents");
        verticalLayout_5 = new QVBoxLayout(tabContents);
        verticalLayout_5->setObjectName("verticalLayout_5");
        contents = new QVBoxLayout();
        contents->setObjectName("contents");

        verticalLayout_5->addLayout(contents);

        tabs->addTab(tabContents, QString());
        tabIndex = new QWidget();
        tabIndex->setObjectName("tabIndex");
        verticalLayout_3 = new QVBoxLayout(tabIndex);
        verticalLayout_3->setObjectName("verticalLayout_3");
        index = new QVBoxLayout();
        index->setObjectName("index");

        verticalLayout_3->addLayout(index);

        tabs->addTab(tabIndex, QString());
        splitter->addWidget(tabs);
        textBrowser = new HelpBrowser(splitter);
        textBrowser->setObjectName("textBrowser");
        splitter->addWidget(textBrowser);

        verticalLayout->addWidget(splitter);


        retranslateUi(HelpDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(HelpDialog);
    } // setupUi

    void retranslateUi(QDialog *HelpDialog)
    {
        HelpDialog->setWindowTitle(QCoreApplication::translate("HelpDialog", "Cppcheck GUI help", nullptr));
        tabs->setTabText(tabs->indexOf(tabContents), QCoreApplication::translate("HelpDialog", "Contents", nullptr));
        tabs->setTabText(tabs->indexOf(tabIndex), QCoreApplication::translate("HelpDialog", "Index", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HelpDialog: public Ui_HelpDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HELPDIALOG_H
