/********************************************************************************
** Form generated from reading UI file 'resultsview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESULTSVIEW_H
#define UI_RESULTSVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "codeeditor.h"
#include "resultstree.h"

QT_BEGIN_NAMESPACE

class Ui_ResultsView
{
public:
    QVBoxLayout *verticalLayout;
    QProgressBar *mProgress;
    QLabel *mLabelCriticalErrors;
    QSplitter *mVerticalSplitter;
    ResultsTree *mTree;
    QTabWidget *tabWidget;
    QWidget *mTabLog;
    QVBoxLayout *verticalLayout_2;
    QListWidget *mListLog;
    QWidget *mTabDetails;
    QVBoxLayout *verticalLayout_3;
    QTextEdit *mDetails;
    CodeEditor *mCode;

    void setupUi(QWidget *ResultsView)
    {
        if (ResultsView->objectName().isEmpty())
            ResultsView->setObjectName("ResultsView");
        ResultsView->resize(459, 391);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ResultsView->sizePolicy().hasHeightForWidth());
        ResultsView->setSizePolicy(sizePolicy);
        ResultsView->setMaximumSize(QSize(16777215, 16777215));
        verticalLayout = new QVBoxLayout(ResultsView);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        mProgress = new QProgressBar(ResultsView);
        mProgress->setObjectName("mProgress");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mProgress->sizePolicy().hasHeightForWidth());
        mProgress->setSizePolicy(sizePolicy1);
        mProgress->setValue(24);

        verticalLayout->addWidget(mProgress);

        mLabelCriticalErrors = new QLabel(ResultsView);
        mLabelCriticalErrors->setObjectName("mLabelCriticalErrors");
        mLabelCriticalErrors->setStyleSheet(QString::fromUtf8("color: red;"));

        verticalLayout->addWidget(mLabelCriticalErrors);

        mVerticalSplitter = new QSplitter(ResultsView);
        mVerticalSplitter->setObjectName("mVerticalSplitter");
        mVerticalSplitter->setOrientation(Qt::Vertical);
        mTree = new ResultsTree(mVerticalSplitter);
        mTree->setObjectName("mTree");
        sizePolicy.setHeightForWidth(mTree->sizePolicy().hasHeightForWidth());
        mTree->setSizePolicy(sizePolicy);
        mTree->setEditTriggers(QAbstractItemView::NoEditTriggers);
        mTree->setSelectionMode(QAbstractItemView::ExtendedSelection);
        mVerticalSplitter->addWidget(mTree);
        tabWidget = new QTabWidget(mVerticalSplitter);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setTabPosition(QTabWidget::South);
        mTabLog = new QWidget();
        mTabLog->setObjectName("mTabLog");
        verticalLayout_2 = new QVBoxLayout(mTabLog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        mListLog = new QListWidget(mTabLog);
        mListLog->setObjectName("mListLog");

        verticalLayout_2->addWidget(mListLog);

        tabWidget->addTab(mTabLog, QString());
        mTabDetails = new QWidget();
        mTabDetails->setObjectName("mTabDetails");
        verticalLayout_3 = new QVBoxLayout(mTabDetails);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        mDetails = new QTextEdit(mTabDetails);
        mDetails->setObjectName("mDetails");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(1);
        sizePolicy2.setHeightForWidth(mDetails->sizePolicy().hasHeightForWidth());
        mDetails->setSizePolicy(sizePolicy2);
        mDetails->setUndoRedoEnabled(false);
        mDetails->setReadOnly(true);

        verticalLayout_3->addWidget(mDetails);

        mCode = new CodeEditor(mTabDetails);
        mCode->setObjectName("mCode");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(4);
        sizePolicy3.setHeightForWidth(mCode->sizePolicy().hasHeightForWidth());
        mCode->setSizePolicy(sizePolicy3);
        mCode->setUndoRedoEnabled(false);
        mCode->setLineWrapMode(QPlainTextEdit::NoWrap);
        mCode->setReadOnly(true);

        verticalLayout_3->addWidget(mCode);

        tabWidget->addTab(mTabDetails, QString());
        mVerticalSplitter->addWidget(tabWidget);

        verticalLayout->addWidget(mVerticalSplitter);

        QWidget::setTabOrder(mTree, mDetails);

        retranslateUi(ResultsView);

        tabWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(ResultsView);
    } // setupUi

    void retranslateUi(QWidget *ResultsView)
    {
        ResultsView->setWindowTitle(QCoreApplication::translate("ResultsView", "Results", nullptr));
        mLabelCriticalErrors->setText(QCoreApplication::translate("ResultsView", "Critical errors", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabLog), QCoreApplication::translate("ResultsView", "Analysis Log", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabDetails), QCoreApplication::translate("ResultsView", "Warning Details", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ResultsView: public Ui_ResultsView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RESULTSVIEW_H
