/****************************************************************************
** Meta object code from reading C++ file 'resultstree.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../gui/resultstree.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'resultstree.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11ResultsTreeE_t {};
} // unnamed namespace

template <> constexpr inline auto ResultsTree::qt_create_metaobjectdata<qt_meta_tag_ZN11ResultsTreeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ResultsTree",
        "resultsHidden",
        "",
        "hidden",
        "checkSelected",
        "selectedItems",
        "treeSelectionChanged",
        "const ResultItem*",
        "selectedItem",
        "suppressIds",
        "ids",
        "showResults",
        "ShowTypes::ShowType",
        "type",
        "show",
        "showCppcheckResults",
        "showClangResults",
        "quickStartApplication",
        "QModelIndex",
        "index",
        "context",
        "application",
        "copy",
        "hideResult",
        "recheckSelectedFiles",
        "hideAllIdResult",
        "suppressSelectedIds",
        "suppressHash",
        "openContainingFolder",
        "currentChanged",
        "current",
        "previous"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resultsHidden'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'checkSelected'
        QtMocHelpers::SignalData<void(QStringList)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Signal 'treeSelectionChanged'
        QtMocHelpers::SignalData<void(const ResultItem *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'suppressIds'
        QtMocHelpers::SignalData<void(QStringList)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 10 },
        }}),
        // Slot 'showResults'
        QtMocHelpers::SlotData<void(ShowTypes::ShowType, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'showCppcheckResults'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'showClangResults'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'quickStartApplication'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'context'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'copy'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'hideResult'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'recheckSelectedFiles'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'hideAllIdResult'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'suppressSelectedIds'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'suppressHash'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openContainingFolder'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'currentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 18, 30 }, { 0x80000000 | 18, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResultsTree, qt_meta_tag_ZN11ResultsTreeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ResultsTree::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ResultsTreeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ResultsTreeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11ResultsTreeE_t>.metaTypes,
    nullptr
} };

void ResultsTree::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResultsTree *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resultsHidden((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->checkSelected((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->treeSelectionChanged((*reinterpret_cast<std::add_pointer_t<const ResultItem*>>(_a[1]))); break;
        case 3: _t->suppressIds((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->showResults((*reinterpret_cast<std::add_pointer_t<ShowTypes::ShowType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->showCppcheckResults((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->showClangResults((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->quickStartApplication((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->context((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->copy(); break;
        case 10: _t->hideResult(); break;
        case 11: _t->recheckSelectedFiles(); break;
        case 12: _t->hideAllIdResult(); break;
        case 13: _t->suppressSelectedIds(); break;
        case 14: _t->suppressHash(); break;
        case 15: _t->openContainingFolder(); break;
        case 16: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResultsTree::*)(bool )>(_a, &ResultsTree::resultsHidden, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsTree::*)(QStringList )>(_a, &ResultsTree::checkSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsTree::*)(const ResultItem * )>(_a, &ResultsTree::treeSelectionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsTree::*)(QStringList )>(_a, &ResultsTree::suppressIds, 3))
            return;
    }
}

const QMetaObject *ResultsTree::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResultsTree::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ResultsTreeE_t>.strings))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int ResultsTree::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void ResultsTree::resultsHidden(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ResultsTree::checkSelected(QStringList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ResultsTree::treeSelectionChanged(const ResultItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ResultsTree::suppressIds(QStringList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
