/********************************************************************************
** Form generated from reading UI file 'libraryeditargdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LIBRARYEDITARGDIALOG_H
#define UI_LIBRARYEDITARGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LibraryEditArgDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QCheckBox *notbool;
    QCheckBox *notnull;
    QCheckBox *notuninit;
    QCheckBox *strz;
    QCheckBox *formatstr;
    QSpacerItem *verticalSpacer_3;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QGridLayout *gridLayout_2;
    QLabel *label_3;
    QComboBox *minsize1type;
    QLabel *label_4;
    QHBoxLayout *horizontalLayout_3;
    QSpinBox *minsize1arg;
    QSpacerItem *horizontalSpacer;
    QLabel *label_5;
    QHBoxLayout *horizontalLayout;
    QSpinBox *minsize1arg2;
    QSpacerItem *horizontalSpacer_2;
    QLabel *andlabel;
    QGridLayout *gridLayout;
    QLabel *minsize2label1;
    QComboBox *minsize2type;
    QLabel *minsize2label2;
    QHBoxLayout *horizontalLayout_5;
    QSpinBox *minsize2arg;
    QSpacerItem *horizontalSpacer_4;
    QLabel *minsize2label3;
    QHBoxLayout *horizontalLayout_4;
    QSpinBox *minsize2arg2;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QLineEdit *valid;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LibraryEditArgDialog)
    {
        if (LibraryEditArgDialog->objectName().isEmpty())
            LibraryEditArgDialog->setObjectName("LibraryEditArgDialog");
        LibraryEditArgDialog->resize(448, 465);
        verticalLayout_3 = new QVBoxLayout(LibraryEditArgDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        notbool = new QCheckBox(LibraryEditArgDialog);
        notbool->setObjectName("notbool");

        verticalLayout_3->addWidget(notbool);

        notnull = new QCheckBox(LibraryEditArgDialog);
        notnull->setObjectName("notnull");

        verticalLayout_3->addWidget(notnull);

        notuninit = new QCheckBox(LibraryEditArgDialog);
        notuninit->setObjectName("notuninit");

        verticalLayout_3->addWidget(notuninit);

        strz = new QCheckBox(LibraryEditArgDialog);
        strz->setObjectName("strz");

        verticalLayout_3->addWidget(strz);

        formatstr = new QCheckBox(LibraryEditArgDialog);
        formatstr->setObjectName("formatstr");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(formatstr->sizePolicy().hasHeightForWidth());
        formatstr->setSizePolicy(sizePolicy);

        verticalLayout_3->addWidget(formatstr);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(LibraryEditArgDialog);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        label_3 = new QLabel(LibraryEditArgDialog);
        label_3->setObjectName("label_3");

        gridLayout_2->addWidget(label_3, 0, 0, 1, 1);

        minsize1type = new QComboBox(LibraryEditArgDialog);
        minsize1type->addItem(QString());
        minsize1type->addItem(QString());
        minsize1type->addItem(QString());
        minsize1type->addItem(QString());
        minsize1type->setObjectName("minsize1type");

        gridLayout_2->addWidget(minsize1type, 0, 1, 1, 1);

        label_4 = new QLabel(LibraryEditArgDialog);
        label_4->setObjectName("label_4");

        gridLayout_2->addWidget(label_4, 1, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        minsize1arg = new QSpinBox(LibraryEditArgDialog);
        minsize1arg->setObjectName("minsize1arg");

        horizontalLayout_3->addWidget(minsize1arg);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        gridLayout_2->addLayout(horizontalLayout_3, 1, 1, 1, 1);

        label_5 = new QLabel(LibraryEditArgDialog);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 2, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        minsize1arg2 = new QSpinBox(LibraryEditArgDialog);
        minsize1arg2->setObjectName("minsize1arg2");

        horizontalLayout->addWidget(minsize1arg2);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        gridLayout_2->addLayout(horizontalLayout, 2, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        andlabel = new QLabel(LibraryEditArgDialog);
        andlabel->setObjectName("andlabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(andlabel->sizePolicy().hasHeightForWidth());
        andlabel->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(andlabel);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        minsize2label1 = new QLabel(LibraryEditArgDialog);
        minsize2label1->setObjectName("minsize2label1");

        gridLayout->addWidget(minsize2label1, 0, 0, 1, 1);

        minsize2type = new QComboBox(LibraryEditArgDialog);
        minsize2type->addItem(QString());
        minsize2type->addItem(QString());
        minsize2type->addItem(QString());
        minsize2type->addItem(QString());
        minsize2type->setObjectName("minsize2type");
        minsize2type->setEnabled(true);

        gridLayout->addWidget(minsize2type, 0, 1, 1, 1);

        minsize2label2 = new QLabel(LibraryEditArgDialog);
        minsize2label2->setObjectName("minsize2label2");

        gridLayout->addWidget(minsize2label2, 1, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        minsize2arg = new QSpinBox(LibraryEditArgDialog);
        minsize2arg->setObjectName("minsize2arg");

        horizontalLayout_5->addWidget(minsize2arg);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);


        gridLayout->addLayout(horizontalLayout_5, 1, 1, 1, 1);

        minsize2label3 = new QLabel(LibraryEditArgDialog);
        minsize2label3->setObjectName("minsize2label3");

        gridLayout->addWidget(minsize2label3, 2, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        minsize2arg2 = new QSpinBox(LibraryEditArgDialog);
        minsize2arg2->setObjectName("minsize2arg2");

        horizontalLayout_4->addWidget(minsize2arg2);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout_4, 2, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);


        verticalLayout_3->addLayout(verticalLayout);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(LibraryEditArgDialog);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        valid = new QLineEdit(LibraryEditArgDialog);
        valid->setObjectName("valid");

        horizontalLayout_2->addWidget(valid);


        verticalLayout_3->addLayout(horizontalLayout_2);

        buttonBox = new QDialogButtonBox(LibraryEditArgDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(buttonBox);


        retranslateUi(LibraryEditArgDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, LibraryEditArgDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, LibraryEditArgDialog, qOverload<>(&QDialog::reject));
        QObject::connect(minsize1type, SIGNAL(currentIndexChanged(int)), LibraryEditArgDialog, SLOT(minsizeChanged()));
        QObject::connect(minsize2type, SIGNAL(currentIndexChanged(int)), LibraryEditArgDialog, SLOT(minsizeChanged()));

        QMetaObject::connectSlotsByName(LibraryEditArgDialog);
    } // setupUi

    void retranslateUi(QDialog *LibraryEditArgDialog)
    {
        LibraryEditArgDialog->setWindowTitle(QCoreApplication::translate("LibraryEditArgDialog", "Edit argument", nullptr));
#if QT_CONFIG(tooltip)
        notbool->setToolTip(QCoreApplication::translate("LibraryEditArgDialog", "<html><head/><body>\n"
"<p>Is bool value allowed? For instance result from comparison or from '!' operator.</p>\n"
"<p>Typically, set this if the argument is a pointer, size, etc.</p>\n"
"<p>Example:</p>\n"
"<pre>    memcmp(x, y, i == 123);   // last argument should not have a bool value</pre>\n"
"</body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        notbool->setText(QCoreApplication::translate("LibraryEditArgDialog", "Not bool", nullptr));
#if QT_CONFIG(tooltip)
        notnull->setToolTip(QCoreApplication::translate("LibraryEditArgDialog", "<html><head/><body>\n"
"<p>Is a null parameter value allowed?</p>\n"
"<p>Typically this should be used on any pointer parameter that does not allow null.</p>\n"
"<p>Example:</p>\n"
"<pre>    strcpy(x,y); // neither x or y is allowed to be null.</pre>\n"
"</body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        notnull->setText(QCoreApplication::translate("LibraryEditArgDialog", "Not null", nullptr));
        notuninit->setText(QCoreApplication::translate("LibraryEditArgDialog", "Not uninit", nullptr));
        strz->setText(QCoreApplication::translate("LibraryEditArgDialog", "String", nullptr));
        formatstr->setText(QCoreApplication::translate("LibraryEditArgDialog", "Format string", nullptr));
        label->setText(QCoreApplication::translate("LibraryEditArgDialog", "Min size of buffer", nullptr));
        label_3->setText(QCoreApplication::translate("LibraryEditArgDialog", "Type", nullptr));
        minsize1type->setItemText(0, QCoreApplication::translate("LibraryEditArgDialog", "None", nullptr));
        minsize1type->setItemText(1, QCoreApplication::translate("LibraryEditArgDialog", "argvalue", nullptr));
        minsize1type->setItemText(2, QCoreApplication::translate("LibraryEditArgDialog", "mul", nullptr));
        minsize1type->setItemText(3, QCoreApplication::translate("LibraryEditArgDialog", "strlen", nullptr));

        label_4->setText(QCoreApplication::translate("LibraryEditArgDialog", "Arg", nullptr));
        label_5->setText(QCoreApplication::translate("LibraryEditArgDialog", "Arg2", nullptr));
        andlabel->setText(QCoreApplication::translate("LibraryEditArgDialog", "and", nullptr));
        minsize2label1->setText(QCoreApplication::translate("LibraryEditArgDialog", "Type", nullptr));
        minsize2type->setItemText(0, QCoreApplication::translate("LibraryEditArgDialog", "None", nullptr));
        minsize2type->setItemText(1, QCoreApplication::translate("LibraryEditArgDialog", "argvalue", nullptr));
        minsize2type->setItemText(2, QCoreApplication::translate("LibraryEditArgDialog", "mul", nullptr));
        minsize2type->setItemText(3, QCoreApplication::translate("LibraryEditArgDialog", "strlen", nullptr));

        minsize2label2->setText(QCoreApplication::translate("LibraryEditArgDialog", "Arg", nullptr));
        minsize2label3->setText(QCoreApplication::translate("LibraryEditArgDialog", "Arg2", nullptr));
        label_2->setText(QCoreApplication::translate("LibraryEditArgDialog", "Valid values", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LibraryEditArgDialog: public Ui_LibraryEditArgDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LIBRARYEDITARGDIALOG_H
