/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "resultsview.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *mActionLicense;
    QAction *mActionAuthors;
    QAction *mActionAbout;
    QAction *mActionAnalyzeFiles;
    QAction *mActionAnalyzeDirectory;
    QAction *mActionReanalyzeModified;
    QAction *mActionReanalyzeAll;
    QAction *mActionStop;
    QAction *mActionSave;
    QAction *mActionQuit;
    QAction *mActionClearResults;
    QAction *mActionSettings;
    QAction *mActionShowStyle;
    QAction *mActionShowErrors;
    QAction *mActionCheckAll;
    QAction *mActionUncheckAll;
    QAction *mActionCollapseAll;
    QAction *mActionExpandAll;
    QAction *mActionToolBarMain;
    QAction *mActionHelpContents;
    QAction *actionToolbar;
    QAction *mActionToolBarView;
    QAction *mActionOpenXML;
    QAction *mActionOpenProjectFile;
    QAction *mActionShowScratchpad;
    QAction *mActionNewProjectFile;
    QAction *mActionViewLog;
    QAction *mActionCloseProjectFile;
    QAction *mActionEditProjectFile;
    QAction *mActionViewStats;
    QAction *mActionShowWarnings;
    QAction *mActionShowPerformance;
    QAction *mActionShowHidden;
    QAction *mActionShowInformation;
    QAction *mActionShowPortability;
    QAction *mActionShowCppcheck;
    QAction *mActionShowClang;
    QAction *mActionToolBarFilter;
    QAction *mActionProjectMRU;
    QAction *mActionPlatformWin32ANSI;
    QAction *mActionPlatformWin32Unicode;
    QAction *mActionPlatformUnix32Bit;
    QAction *mActionPlatformUnix64Bit;
    QAction *mActionPlatformWin64;
    QAction *mActionPlatforms;
    QAction *mActionCpp11;
    QAction *mActionC99;
    QAction *mActionPosix;
    QAction *mActionC11;
    QAction *mActionC89;
    QAction *mActionCpp03;
    QAction *mActionPrint;
    QAction *mActionPrintPreview;
    QAction *mActionLibraryEditor;
    QAction *mActionAutoDetectLanguage;
    QAction *mActionEnforceCpp;
    QAction *mActionEnforceC;
    QAction *mActionCpp14;
    QAction *mActionCheckLibrary;
    QAction *mActionCheckConfiguration;
    QAction *mActionCpp17;
    QAction *mActionCpp20;
    QAction *mActionComplianceReport;
    QAction *actionReport;
    QAction *mActionReportNormal;
    QAction *mActionReportMisraC;
    QAction *mActionReportMisraCpp2008;
    QAction *mActionReportCertC;
    QAction *mActionReportCertCpp;
    QAction *mActionReportMisraCpp2023;
    QAction *mActionReportAutosar;
    QAction *mActionEULA;
    QAction *mActionShowThreadDetails;
    QWidget *mCentral;
    QVBoxLayout *verticalLayout;
    ResultsView *mResults;
    QHBoxLayout *mLayoutInformation;
    QLabel *mLabelInformation;
    QPushButton *mButtonHideInformation;
    QSpacerItem *horizontalSpacer;
    QMenuBar *mMenuBar;
    QMenu *mMenuFile;
    QMenu *mMenuView;
    QMenu *mMenuToolbars;
    QMenu *menuReport;
    QMenu *mMenuHelp;
    QMenu *mMenuAnalyze;
    QMenu *menuCpp_standard;
    QMenu *menuC_standard;
    QMenu *mMenuEdit;
    QToolBar *mToolBarMain;
    QToolBar *mToolBarView;
    QToolBar *mToolBarFilter;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(640, 480);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MainWindow->sizePolicy().hasHeightForWidth());
        MainWindow->setSizePolicy(sizePolicy);
        MainWindow->setMinimumSize(QSize(640, 480));
        MainWindow->setIconSize(QSize(22, 22));
        mActionLicense = new QAction(MainWindow);
        mActionLicense->setObjectName("mActionLicense");
        mActionAuthors = new QAction(MainWindow);
        mActionAuthors->setObjectName("mActionAuthors");
        mActionAbout = new QAction(MainWindow);
        mActionAbout->setObjectName("mActionAbout");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/help-browser.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionAbout->setIcon(icon);
        mActionAnalyzeFiles = new QAction(MainWindow);
        mActionAnalyzeFiles->setObjectName("mActionAnalyzeFiles");
        mActionAnalyzeDirectory = new QAction(MainWindow);
        mActionAnalyzeDirectory->setObjectName("mActionAnalyzeDirectory");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/cppcheck-gui.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionAnalyzeDirectory->setIcon(icon1);
        mActionReanalyzeModified = new QAction(MainWindow);
        mActionReanalyzeModified->setObjectName("mActionReanalyzeModified");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/view-refresh.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionReanalyzeModified->setIcon(icon2);
        mActionReanalyzeAll = new QAction(MainWindow);
        mActionReanalyzeAll->setObjectName("mActionReanalyzeAll");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/view-recheck.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionReanalyzeAll->setIcon(icon3);
        mActionStop = new QAction(MainWindow);
        mActionStop->setObjectName("mActionStop");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/images/process-stop.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionStop->setIcon(icon4);
        mActionSave = new QAction(MainWindow);
        mActionSave->setObjectName("mActionSave");
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/images/media-floppy.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionSave->setIcon(icon5);
        mActionQuit = new QAction(MainWindow);
        mActionQuit->setObjectName("mActionQuit");
        mActionClearResults = new QAction(MainWindow);
        mActionClearResults->setObjectName("mActionClearResults");
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/images/edit-clear.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionClearResults->setIcon(icon6);
        mActionSettings = new QAction(MainWindow);
        mActionSettings->setObjectName("mActionSettings");
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/images/preferences-system.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionSettings->setIcon(icon7);
        mActionShowStyle = new QAction(MainWindow);
        mActionShowStyle->setObjectName("mActionShowStyle");
        mActionShowStyle->setCheckable(true);
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/images/applications-development.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowStyle->setIcon(icon8);
        mActionShowErrors = new QAction(MainWindow);
        mActionShowErrors->setObjectName("mActionShowErrors");
        mActionShowErrors->setCheckable(true);
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/images/showerrors.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowErrors->setIcon(icon9);
        mActionCheckAll = new QAction(MainWindow);
        mActionCheckAll->setObjectName("mActionCheckAll");
        mActionUncheckAll = new QAction(MainWindow);
        mActionUncheckAll->setObjectName("mActionUncheckAll");
        mActionCollapseAll = new QAction(MainWindow);
        mActionCollapseAll->setObjectName("mActionCollapseAll");
        mActionExpandAll = new QAction(MainWindow);
        mActionExpandAll->setObjectName("mActionExpandAll");
        mActionToolBarMain = new QAction(MainWindow);
        mActionToolBarMain->setObjectName("mActionToolBarMain");
        mActionToolBarMain->setCheckable(true);
        mActionHelpContents = new QAction(MainWindow);
        mActionHelpContents->setObjectName("mActionHelpContents");
        actionToolbar = new QAction(MainWindow);
        actionToolbar->setObjectName("actionToolbar");
        mActionToolBarView = new QAction(MainWindow);
        mActionToolBarView->setObjectName("mActionToolBarView");
        mActionToolBarView->setCheckable(true);
        mActionOpenXML = new QAction(MainWindow);
        mActionOpenXML->setObjectName("mActionOpenXML");
        mActionOpenProjectFile = new QAction(MainWindow);
        mActionOpenProjectFile->setObjectName("mActionOpenProjectFile");
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/images/openproject.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionOpenProjectFile->setIcon(icon10);
        mActionShowScratchpad = new QAction(MainWindow);
        mActionShowScratchpad->setObjectName("mActionShowScratchpad");
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/images/scratchpad.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowScratchpad->setIcon(icon11);
        mActionNewProjectFile = new QAction(MainWindow);
        mActionNewProjectFile->setObjectName("mActionNewProjectFile");
        mActionViewLog = new QAction(MainWindow);
        mActionViewLog->setObjectName("mActionViewLog");
        mActionCloseProjectFile = new QAction(MainWindow);
        mActionCloseProjectFile->setObjectName("mActionCloseProjectFile");
        mActionCloseProjectFile->setEnabled(false);
        mActionEditProjectFile = new QAction(MainWindow);
        mActionEditProjectFile->setObjectName("mActionEditProjectFile");
        mActionEditProjectFile->setEnabled(false);
        mActionViewStats = new QAction(MainWindow);
        mActionViewStats->setObjectName("mActionViewStats");
        mActionViewStats->setEnabled(false);
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/images/text-x-generic.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionViewStats->setIcon(icon12);
        mActionShowWarnings = new QAction(MainWindow);
        mActionShowWarnings->setObjectName("mActionShowWarnings");
        mActionShowWarnings->setCheckable(true);
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/images/showwarnings.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowWarnings->setIcon(icon13);
        mActionShowPerformance = new QAction(MainWindow);
        mActionShowPerformance->setObjectName("mActionShowPerformance");
        mActionShowPerformance->setCheckable(true);
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/images/showperformance.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowPerformance->setIcon(icon14);
        mActionShowHidden = new QAction(MainWindow);
        mActionShowHidden->setObjectName("mActionShowHidden");
        mActionShowHidden->setEnabled(false);
        mActionShowInformation = new QAction(MainWindow);
        mActionShowInformation->setObjectName("mActionShowInformation");
        mActionShowInformation->setCheckable(true);
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/images/dialog-information.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowInformation->setIcon(icon15);
        mActionShowPortability = new QAction(MainWindow);
        mActionShowPortability->setObjectName("mActionShowPortability");
        mActionShowPortability->setCheckable(true);
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/images/applications-system.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowPortability->setIcon(icon16);
        mActionShowCppcheck = new QAction(MainWindow);
        mActionShowCppcheck->setObjectName("mActionShowCppcheck");
        mActionShowCppcheck->setCheckable(true);
        mActionShowCppcheck->setIcon(icon1);
        mActionShowClang = new QAction(MainWindow);
        mActionShowClang->setObjectName("mActionShowClang");
        mActionShowClang->setCheckable(true);
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/images/llvm-dragon.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mActionShowClang->setIcon(icon17);
        mActionToolBarFilter = new QAction(MainWindow);
        mActionToolBarFilter->setObjectName("mActionToolBarFilter");
        mActionToolBarFilter->setCheckable(true);
        mActionProjectMRU = new QAction(MainWindow);
        mActionProjectMRU->setObjectName("mActionProjectMRU");
        mActionProjectMRU->setText(QString::fromUtf8("Project &MRU placeholder"));
        mActionProjectMRU->setVisible(true);
        mActionPlatformWin32ANSI = new QAction(MainWindow);
        mActionPlatformWin32ANSI->setObjectName("mActionPlatformWin32ANSI");
        mActionPlatformWin32ANSI->setCheckable(true);
        mActionPlatformWin32Unicode = new QAction(MainWindow);
        mActionPlatformWin32Unicode->setObjectName("mActionPlatformWin32Unicode");
        mActionPlatformWin32Unicode->setCheckable(true);
        mActionPlatformUnix32Bit = new QAction(MainWindow);
        mActionPlatformUnix32Bit->setObjectName("mActionPlatformUnix32Bit");
        mActionPlatformUnix32Bit->setCheckable(true);
        mActionPlatformUnix64Bit = new QAction(MainWindow);
        mActionPlatformUnix64Bit->setObjectName("mActionPlatformUnix64Bit");
        mActionPlatformUnix64Bit->setCheckable(true);
        mActionPlatformWin64 = new QAction(MainWindow);
        mActionPlatformWin64->setObjectName("mActionPlatformWin64");
        mActionPlatformWin64->setCheckable(true);
        mActionPlatforms = new QAction(MainWindow);
        mActionPlatforms->setObjectName("mActionPlatforms");
        mActionPlatforms->setCheckable(false);
        mActionPlatforms->setVisible(false);
        mActionCpp11 = new QAction(MainWindow);
        mActionCpp11->setObjectName("mActionCpp11");
        mActionCpp11->setCheckable(true);
        mActionCpp11->setChecked(false);
        mActionC99 = new QAction(MainWindow);
        mActionC99->setObjectName("mActionC99");
        mActionC99->setCheckable(true);
        mActionC99->setChecked(true);
        mActionPosix = new QAction(MainWindow);
        mActionPosix->setObjectName("mActionPosix");
        mActionPosix->setCheckable(true);
        mActionC11 = new QAction(MainWindow);
        mActionC11->setObjectName("mActionC11");
        mActionC11->setCheckable(true);
        mActionC89 = new QAction(MainWindow);
        mActionC89->setObjectName("mActionC89");
        mActionC89->setCheckable(true);
        mActionCpp03 = new QAction(MainWindow);
        mActionCpp03->setObjectName("mActionCpp03");
        mActionCpp03->setCheckable(true);
        mActionPrint = new QAction(MainWindow);
        mActionPrint->setObjectName("mActionPrint");
        mActionPrintPreview = new QAction(MainWindow);
        mActionPrintPreview->setObjectName("mActionPrintPreview");
        mActionLibraryEditor = new QAction(MainWindow);
        mActionLibraryEditor->setObjectName("mActionLibraryEditor");
        mActionAutoDetectLanguage = new QAction(MainWindow);
        mActionAutoDetectLanguage->setObjectName("mActionAutoDetectLanguage");
        mActionAutoDetectLanguage->setCheckable(true);
        mActionEnforceCpp = new QAction(MainWindow);
        mActionEnforceCpp->setObjectName("mActionEnforceCpp");
        mActionEnforceCpp->setCheckable(true);
        mActionEnforceC = new QAction(MainWindow);
        mActionEnforceC->setObjectName("mActionEnforceC");
        mActionEnforceC->setCheckable(true);
        mActionCpp14 = new QAction(MainWindow);
        mActionCpp14->setObjectName("mActionCpp14");
        mActionCpp14->setCheckable(true);
        mActionCpp14->setChecked(false);
        mActionCheckLibrary = new QAction(MainWindow);
        mActionCheckLibrary->setObjectName("mActionCheckLibrary");
        mActionCheckLibrary->setEnabled(false);
        mActionCheckConfiguration = new QAction(MainWindow);
        mActionCheckConfiguration->setObjectName("mActionCheckConfiguration");
        mActionCheckConfiguration->setEnabled(false);
        mActionCpp17 = new QAction(MainWindow);
        mActionCpp17->setObjectName("mActionCpp17");
        mActionCpp17->setCheckable(true);
        mActionCpp20 = new QAction(MainWindow);
        mActionCpp20->setObjectName("mActionCpp20");
        mActionCpp20->setCheckable(true);
        mActionCpp20->setChecked(true);
        mActionComplianceReport = new QAction(MainWindow);
        mActionComplianceReport->setObjectName("mActionComplianceReport");
        actionReport = new QAction(MainWindow);
        actionReport->setObjectName("actionReport");
        mActionReportNormal = new QAction(MainWindow);
        mActionReportNormal->setObjectName("mActionReportNormal");
        mActionReportNormal->setCheckable(true);
        mActionReportMisraC = new QAction(MainWindow);
        mActionReportMisraC->setObjectName("mActionReportMisraC");
        mActionReportMisraC->setCheckable(true);
        mActionReportMisraCpp2008 = new QAction(MainWindow);
        mActionReportMisraCpp2008->setObjectName("mActionReportMisraCpp2008");
        mActionReportMisraCpp2008->setCheckable(true);
        mActionReportCertC = new QAction(MainWindow);
        mActionReportCertC->setObjectName("mActionReportCertC");
        mActionReportCertC->setCheckable(true);
        mActionReportCertCpp = new QAction(MainWindow);
        mActionReportCertCpp->setObjectName("mActionReportCertCpp");
        mActionReportCertCpp->setCheckable(true);
        mActionReportMisraCpp2023 = new QAction(MainWindow);
        mActionReportMisraCpp2023->setObjectName("mActionReportMisraCpp2023");
        mActionReportMisraCpp2023->setCheckable(true);
        mActionReportAutosar = new QAction(MainWindow);
        mActionReportAutosar->setObjectName("mActionReportAutosar");
        mActionReportAutosar->setCheckable(true);
        mActionEULA = new QAction(MainWindow);
        mActionEULA->setObjectName("mActionEULA");
        mActionShowThreadDetails = new QAction(MainWindow);
        mActionShowThreadDetails->setObjectName("mActionShowThreadDetails");
        mCentral = new QWidget(MainWindow);
        mCentral->setObjectName("mCentral");
        sizePolicy.setHeightForWidth(mCentral->sizePolicy().hasHeightForWidth());
        mCentral->setSizePolicy(sizePolicy);
        mCentral->setMinimumSize(QSize(0, 0));
        mCentral->setMaximumSize(QSize(16777215, 16777215));
        verticalLayout = new QVBoxLayout(mCentral);
        verticalLayout->setObjectName("verticalLayout");
        mResults = new ResultsView(mCentral);
        mResults->setObjectName("mResults");
        sizePolicy.setHeightForWidth(mResults->sizePolicy().hasHeightForWidth());
        mResults->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(mResults);

        mLayoutInformation = new QHBoxLayout();
        mLayoutInformation->setObjectName("mLayoutInformation");
        mLayoutInformation->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        mLabelInformation = new QLabel(mCentral);
        mLabelInformation->setObjectName("mLabelInformation");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mLabelInformation->sizePolicy().hasHeightForWidth());
        mLabelInformation->setSizePolicy(sizePolicy1);
        mLabelInformation->setMaximumSize(QSize(16777215, 25));
        mLabelInformation->setTextFormat(Qt::TextFormat::RichText);
        mLabelInformation->setOpenExternalLinks(true);
        mLabelInformation->setTextInteractionFlags(Qt::TextInteractionFlag::TextBrowserInteraction);

        mLayoutInformation->addWidget(mLabelInformation);

        mButtonHideInformation = new QPushButton(mCentral);
        mButtonHideInformation->setObjectName("mButtonHideInformation");

        mLayoutInformation->addWidget(mButtonHideInformation);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        mLayoutInformation->addItem(horizontalSpacer);


        verticalLayout->addLayout(mLayoutInformation);

        MainWindow->setCentralWidget(mCentral);
        mMenuBar = new QMenuBar(MainWindow);
        mMenuBar->setObjectName("mMenuBar");
        mMenuBar->setGeometry(QRect(0, 0, 640, 21));
        mMenuFile = new QMenu(mMenuBar);
        mMenuFile->setObjectName("mMenuFile");
        mMenuView = new QMenu(mMenuBar);
        mMenuView->setObjectName("mMenuView");
        mMenuToolbars = new QMenu(mMenuView);
        mMenuToolbars->setObjectName("mMenuToolbars");
        menuReport = new QMenu(mMenuView);
        menuReport->setObjectName("menuReport");
        mMenuHelp = new QMenu(mMenuBar);
        mMenuHelp->setObjectName("mMenuHelp");
        mMenuAnalyze = new QMenu(mMenuBar);
        mMenuAnalyze->setObjectName("mMenuAnalyze");
        menuCpp_standard = new QMenu(mMenuAnalyze);
        menuCpp_standard->setObjectName("menuCpp_standard");
        menuC_standard = new QMenu(mMenuAnalyze);
        menuC_standard->setObjectName("menuC_standard");
        mMenuEdit = new QMenu(mMenuBar);
        mMenuEdit->setObjectName("mMenuEdit");
        MainWindow->setMenuBar(mMenuBar);
        mToolBarMain = new QToolBar(MainWindow);
        mToolBarMain->setObjectName("mToolBarMain");
        MainWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, mToolBarMain);
        mToolBarView = new QToolBar(MainWindow);
        mToolBarView->setObjectName("mToolBarView");
        MainWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, mToolBarView);
        mToolBarFilter = new QToolBar(MainWindow);
        mToolBarFilter->setObjectName("mToolBarFilter");
        MainWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, mToolBarFilter);

        mMenuBar->addAction(mMenuFile->menuAction());
        mMenuBar->addAction(mMenuEdit->menuAction());
        mMenuBar->addAction(mMenuView->menuAction());
        mMenuBar->addAction(mMenuAnalyze->menuAction());
        mMenuBar->addAction(mMenuHelp->menuAction());
        mMenuFile->addAction(mActionOpenXML);
        mMenuFile->addSeparator();
        mMenuFile->addAction(mActionNewProjectFile);
        mMenuFile->addAction(mActionOpenProjectFile);
        mMenuFile->addAction(mActionEditProjectFile);
        mMenuFile->addAction(mActionCloseProjectFile);
        mMenuFile->addSeparator();
        mMenuFile->addAction(mActionProjectMRU);
        mMenuFile->addAction(mActionSave);
        mMenuFile->addAction(mActionComplianceReport);
        mMenuFile->addSeparator();
        mMenuFile->addAction(mActionPrintPreview);
        mMenuFile->addAction(mActionPrint);
        mMenuFile->addSeparator();
        mMenuFile->addAction(mActionQuit);
        mMenuView->addAction(mMenuToolbars->menuAction());
        mMenuView->addSeparator();
        mMenuView->addAction(menuReport->menuAction());
        mMenuView->addSeparator();
        mMenuView->addAction(mActionShowErrors);
        mMenuView->addAction(mActionShowWarnings);
        mMenuView->addAction(mActionShowStyle);
        mMenuView->addAction(mActionShowPortability);
        mMenuView->addAction(mActionShowPerformance);
        mMenuView->addAction(mActionShowInformation);
        mMenuView->addAction(mActionCheckAll);
        mMenuView->addAction(mActionUncheckAll);
        mMenuView->addSeparator();
        mMenuView->addAction(mActionCollapseAll);
        mMenuView->addAction(mActionExpandAll);
        mMenuView->addSeparator();
        mMenuView->addAction(mActionShowHidden);
        mMenuView->addSeparator();
        mMenuView->addAction(mActionShowScratchpad);
        mMenuView->addAction(mActionViewStats);
        mMenuView->addAction(mActionLibraryEditor);
        mMenuView->addAction(mActionShowThreadDetails);
        mMenuToolbars->addAction(mActionToolBarMain);
        mMenuToolbars->addAction(mActionToolBarView);
        mMenuToolbars->addAction(mActionToolBarFilter);
        menuReport->addAction(mActionReportNormal);
        menuReport->addAction(mActionReportAutosar);
        menuReport->addAction(mActionReportCertC);
        menuReport->addAction(mActionReportCertCpp);
        menuReport->addAction(mActionReportMisraC);
        menuReport->addAction(mActionReportMisraCpp2008);
        menuReport->addAction(mActionReportMisraCpp2023);
        mMenuHelp->addAction(mActionHelpContents);
        mMenuHelp->addAction(mActionLicense);
        mMenuHelp->addAction(mActionAuthors);
        mMenuHelp->addAction(mActionEULA);
        mMenuHelp->addSeparator();
        mMenuHelp->addAction(mActionAbout);
        mMenuAnalyze->addAction(mActionAnalyzeFiles);
        mMenuAnalyze->addAction(mActionAnalyzeDirectory);
        mMenuAnalyze->addAction(mActionReanalyzeModified);
        mMenuAnalyze->addAction(mActionReanalyzeAll);
        mMenuAnalyze->addAction(mActionCheckLibrary);
        mMenuAnalyze->addAction(mActionCheckConfiguration);
        mMenuAnalyze->addAction(mActionStop);
        mMenuAnalyze->addSeparator();
        mMenuAnalyze->addAction(menuC_standard->menuAction());
        mMenuAnalyze->addAction(menuCpp_standard->menuAction());
        mMenuAnalyze->addSeparator();
        mMenuAnalyze->addAction(mActionAutoDetectLanguage);
        mMenuAnalyze->addAction(mActionEnforceCpp);
        mMenuAnalyze->addAction(mActionEnforceC);
        mMenuAnalyze->addSeparator();
        mMenuAnalyze->addAction(mActionPlatforms);
        menuCpp_standard->addAction(mActionCpp03);
        menuCpp_standard->addAction(mActionCpp11);
        menuCpp_standard->addAction(mActionCpp14);
        menuCpp_standard->addAction(mActionCpp17);
        menuCpp_standard->addAction(mActionCpp20);
        menuC_standard->addAction(mActionC89);
        menuC_standard->addAction(mActionC99);
        menuC_standard->addAction(mActionC11);
        mMenuEdit->addAction(mActionClearResults);
        mMenuEdit->addAction(mActionSettings);
        mToolBarMain->addAction(mActionOpenProjectFile);
        mToolBarMain->addAction(mActionSave);
        mToolBarMain->addAction(mActionReanalyzeAll);
        mToolBarMain->addAction(mActionStop);
        mToolBarMain->addAction(mActionClearResults);
        mToolBarMain->addAction(mActionSettings);
        mToolBarMain->addAction(mActionAbout);
        mToolBarView->addAction(mActionShowErrors);
        mToolBarView->addAction(mActionShowWarnings);
        mToolBarView->addAction(mActionShowStyle);
        mToolBarView->addAction(mActionShowPortability);
        mToolBarView->addAction(mActionShowPerformance);
        mToolBarView->addAction(mActionShowInformation);
        mToolBarView->addAction(mActionShowCppcheck);
        mToolBarView->addAction(mActionShowClang);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Cppcheck", nullptr));
        mActionLicense->setText(QCoreApplication::translate("MainWindow", "&License...", nullptr));
        mActionAuthors->setText(QCoreApplication::translate("MainWindow", "A&uthors...", nullptr));
        mActionAbout->setText(QCoreApplication::translate("MainWindow", "&About...", nullptr));
        mActionAnalyzeFiles->setText(QCoreApplication::translate("MainWindow", "&Files...", nullptr));
        mActionAnalyzeFiles->setIconText(QCoreApplication::translate("MainWindow", "Analyze files", nullptr));
#if QT_CONFIG(tooltip)
        mActionAnalyzeFiles->setToolTip(QCoreApplication::translate("MainWindow", "Analyze files", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        mActionAnalyzeFiles->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+F", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionAnalyzeDirectory->setText(QCoreApplication::translate("MainWindow", "&Directory...", nullptr));
        mActionAnalyzeDirectory->setIconText(QCoreApplication::translate("MainWindow", "Analyze directory", nullptr));
#if QT_CONFIG(tooltip)
        mActionAnalyzeDirectory->setToolTip(QCoreApplication::translate("MainWindow", "Analyze directory", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        mActionAnalyzeDirectory->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+D", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionReanalyzeModified->setText(QCoreApplication::translate("MainWindow", "&Reanalyze modified files", nullptr));
#if QT_CONFIG(shortcut)
        mActionReanalyzeModified->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionReanalyzeAll->setText(QCoreApplication::translate("MainWindow", "Reanal&yze all files", nullptr));
        mActionStop->setText(QCoreApplication::translate("MainWindow", "&Stop", nullptr));
        mActionStop->setIconText(QCoreApplication::translate("MainWindow", "Stop analysis", nullptr));
#if QT_CONFIG(tooltip)
        mActionStop->setToolTip(QCoreApplication::translate("MainWindow", "Stop analysis", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        mActionStop->setShortcut(QCoreApplication::translate("MainWindow", "Esc", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionSave->setText(QCoreApplication::translate("MainWindow", "&Save results to file...", nullptr));
#if QT_CONFIG(shortcut)
        mActionSave->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionQuit->setText(QCoreApplication::translate("MainWindow", "&Quit", nullptr));
#if QT_CONFIG(shortcut)
        mActionQuit->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionClearResults->setText(QCoreApplication::translate("MainWindow", "&Clear results", nullptr));
        mActionSettings->setText(QCoreApplication::translate("MainWindow", "&Preferences", nullptr));
        mActionShowStyle->setText(QCoreApplication::translate("MainWindow", "Style war&nings", nullptr));
        mActionShowStyle->setIconText(QCoreApplication::translate("MainWindow", "Show style warnings", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowStyle->setToolTip(QCoreApplication::translate("MainWindow", "Show style warnings", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionShowErrors->setText(QCoreApplication::translate("MainWindow", "E&rrors", nullptr));
        mActionShowErrors->setIconText(QCoreApplication::translate("MainWindow", "Show errors", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowErrors->setToolTip(QCoreApplication::translate("MainWindow", "Show errors", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionCheckAll->setText(QCoreApplication::translate("MainWindow", "&Check all", nullptr));
        mActionUncheckAll->setText(QCoreApplication::translate("MainWindow", "&Uncheck all", nullptr));
        mActionCollapseAll->setText(QCoreApplication::translate("MainWindow", "Collapse &all", nullptr));
        mActionExpandAll->setText(QCoreApplication::translate("MainWindow", "&Expand all", nullptr));
        mActionToolBarMain->setText(QCoreApplication::translate("MainWindow", "&Standard", nullptr));
#if QT_CONFIG(tooltip)
        mActionToolBarMain->setToolTip(QCoreApplication::translate("MainWindow", "Standard items", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionHelpContents->setText(QCoreApplication::translate("MainWindow", "&Contents", nullptr));
#if QT_CONFIG(statustip)
        mActionHelpContents->setStatusTip(QCoreApplication::translate("MainWindow", "Open the help contents", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        mActionHelpContents->setShortcut(QCoreApplication::translate("MainWindow", "F1", nullptr));
#endif // QT_CONFIG(shortcut)
        actionToolbar->setText(QCoreApplication::translate("MainWindow", "Toolbar", nullptr));
        mActionToolBarView->setText(QCoreApplication::translate("MainWindow", "&Categories", nullptr));
#if QT_CONFIG(tooltip)
        mActionToolBarView->setToolTip(QCoreApplication::translate("MainWindow", "Error categories", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionOpenXML->setText(QCoreApplication::translate("MainWindow", "&Open XML...", nullptr));
        mActionOpenProjectFile->setText(QCoreApplication::translate("MainWindow", "Open P&roject File...", nullptr));
#if QT_CONFIG(shortcut)
        mActionOpenProjectFile->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Shift+O", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionShowScratchpad->setText(QCoreApplication::translate("MainWindow", "Sh&ow Scratchpad...", nullptr));
        mActionNewProjectFile->setText(QCoreApplication::translate("MainWindow", "&New Project File...", nullptr));
#if QT_CONFIG(shortcut)
        mActionNewProjectFile->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Shift+N", nullptr));
#endif // QT_CONFIG(shortcut)
        mActionViewLog->setText(QCoreApplication::translate("MainWindow", "&Log View", nullptr));
#if QT_CONFIG(tooltip)
        mActionViewLog->setToolTip(QCoreApplication::translate("MainWindow", "Log View", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionCloseProjectFile->setText(QCoreApplication::translate("MainWindow", "C&lose Project File", nullptr));
        mActionEditProjectFile->setText(QCoreApplication::translate("MainWindow", "&Edit Project File...", nullptr));
        mActionViewStats->setText(QCoreApplication::translate("MainWindow", "&Statistics", nullptr));
        mActionShowWarnings->setText(QCoreApplication::translate("MainWindow", "&Warnings", nullptr));
        mActionShowWarnings->setIconText(QCoreApplication::translate("MainWindow", "Show warnings", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowWarnings->setToolTip(QCoreApplication::translate("MainWindow", "Show warnings", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionShowPerformance->setText(QCoreApplication::translate("MainWindow", "Per&formance warnings", nullptr));
        mActionShowPerformance->setIconText(QCoreApplication::translate("MainWindow", "Show performance warnings", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowPerformance->setToolTip(QCoreApplication::translate("MainWindow", "Show performance warnings", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionShowHidden->setText(QCoreApplication::translate("MainWindow", "Show &hidden", nullptr));
        mActionShowInformation->setText(QCoreApplication::translate("MainWindow", "&Information", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowInformation->setToolTip(QCoreApplication::translate("MainWindow", "Show information messages", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionShowPortability->setText(QCoreApplication::translate("MainWindow", "&Portability", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowPortability->setToolTip(QCoreApplication::translate("MainWindow", "Show portability warnings", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionShowCppcheck->setText(QCoreApplication::translate("MainWindow", "Cppcheck", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowCppcheck->setToolTip(QCoreApplication::translate("MainWindow", "Show Cppcheck results", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionShowClang->setText(QCoreApplication::translate("MainWindow", "Clang", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowClang->setToolTip(QCoreApplication::translate("MainWindow", "Show Clang results", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionToolBarFilter->setText(QCoreApplication::translate("MainWindow", "&Filter", nullptr));
#if QT_CONFIG(tooltip)
        mActionToolBarFilter->setToolTip(QCoreApplication::translate("MainWindow", "Filter results", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionPlatformWin32ANSI->setText(QCoreApplication::translate("MainWindow", "Windows 32-bit ANSI", nullptr));
        mActionPlatformWin32Unicode->setText(QCoreApplication::translate("MainWindow", "Windows 32-bit Unicode", nullptr));
        mActionPlatformUnix32Bit->setText(QCoreApplication::translate("MainWindow", "Unix 32-bit", nullptr));
        mActionPlatformUnix64Bit->setText(QCoreApplication::translate("MainWindow", "Unix 64-bit", nullptr));
        mActionPlatformWin64->setText(QCoreApplication::translate("MainWindow", "Windows 64-bit", nullptr));
        mActionPlatforms->setText(QCoreApplication::translate("MainWindow", "P&latforms", nullptr));
        mActionCpp11->setText(QCoreApplication::translate("MainWindow", "C++&11", nullptr));
        mActionC99->setText(QCoreApplication::translate("MainWindow", "C&99", nullptr));
        mActionPosix->setText(QCoreApplication::translate("MainWindow", "&Posix", nullptr));
        mActionC11->setText(QCoreApplication::translate("MainWindow", "C&11", nullptr));
        mActionC89->setText(QCoreApplication::translate("MainWindow", "&C89", nullptr));
        mActionCpp03->setText(QCoreApplication::translate("MainWindow", "&C++03", nullptr));
        mActionPrint->setText(QCoreApplication::translate("MainWindow", "&Print...", nullptr));
#if QT_CONFIG(tooltip)
        mActionPrint->setToolTip(QCoreApplication::translate("MainWindow", "Print the Current Report", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionPrintPreview->setText(QCoreApplication::translate("MainWindow", "Print Pre&view...", nullptr));
#if QT_CONFIG(tooltip)
        mActionPrintPreview->setToolTip(QCoreApplication::translate("MainWindow", "Open a Print Preview Dialog for the Current Results", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionLibraryEditor->setText(QCoreApplication::translate("MainWindow", "&Library Editor...", nullptr));
#if QT_CONFIG(tooltip)
        mActionLibraryEditor->setToolTip(QCoreApplication::translate("MainWindow", "Open library editor", nullptr));
#endif // QT_CONFIG(tooltip)
        mActionAutoDetectLanguage->setText(QCoreApplication::translate("MainWindow", "&Auto-detect language", nullptr));
        mActionEnforceCpp->setText(QCoreApplication::translate("MainWindow", "&Enforce C++", nullptr));
        mActionEnforceC->setText(QCoreApplication::translate("MainWindow", "E&nforce C", nullptr));
        mActionCpp14->setText(QCoreApplication::translate("MainWindow", "C++14", nullptr));
        mActionCheckLibrary->setText(QCoreApplication::translate("MainWindow", "Reanalyze and check library", nullptr));
        mActionCheckConfiguration->setText(QCoreApplication::translate("MainWindow", "Check configuration (defines, includes)", nullptr));
        mActionCpp17->setText(QCoreApplication::translate("MainWindow", "C++17", nullptr));
        mActionCpp20->setText(QCoreApplication::translate("MainWindow", "C++20", nullptr));
        mActionComplianceReport->setText(QCoreApplication::translate("MainWindow", "Compliance report...", nullptr));
        actionReport->setText(QCoreApplication::translate("MainWindow", "Report", nullptr));
        mActionReportNormal->setText(QCoreApplication::translate("MainWindow", "Normal", nullptr));
        mActionReportMisraC->setText(QCoreApplication::translate("MainWindow", "Misra C", nullptr));
        mActionReportMisraCpp2008->setText(QCoreApplication::translate("MainWindow", "Misra C++ 2008", nullptr));
        mActionReportCertC->setText(QCoreApplication::translate("MainWindow", "Cert C", nullptr));
        mActionReportCertCpp->setText(QCoreApplication::translate("MainWindow", "Cert C++", nullptr));
        mActionReportMisraCpp2023->setText(QCoreApplication::translate("MainWindow", "Misra C++ 2023", nullptr));
        mActionReportAutosar->setText(QCoreApplication::translate("MainWindow", "Autosar", nullptr));
        mActionEULA->setText(QCoreApplication::translate("MainWindow", "EULA...", nullptr));
        mActionShowThreadDetails->setText(QCoreApplication::translate("MainWindow", "Thread Details", nullptr));
#if QT_CONFIG(tooltip)
        mActionShowThreadDetails->setToolTip(QCoreApplication::translate("MainWindow", "Show thread details", nullptr));
#endif // QT_CONFIG(tooltip)
        mLabelInformation->setText(QCoreApplication::translate("MainWindow", "Checking for updates", nullptr));
        mButtonHideInformation->setText(QCoreApplication::translate("MainWindow", "Hide", nullptr));
        mMenuFile->setTitle(QCoreApplication::translate("MainWindow", "&File", nullptr));
        mMenuView->setTitle(QCoreApplication::translate("MainWindow", "&View", nullptr));
        mMenuToolbars->setTitle(QCoreApplication::translate("MainWindow", "&Toolbars", nullptr));
        menuReport->setTitle(QCoreApplication::translate("MainWindow", "Report", nullptr));
        mMenuHelp->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
        mMenuAnalyze->setTitle(QCoreApplication::translate("MainWindow", "A&nalyze", nullptr));
        menuCpp_standard->setTitle(QCoreApplication::translate("MainWindow", "C++ standard", nullptr));
        menuC_standard->setTitle(QCoreApplication::translate("MainWindow", "&C standard", nullptr));
        mMenuEdit->setTitle(QCoreApplication::translate("MainWindow", "&Edit", nullptr));
        mToolBarMain->setWindowTitle(QCoreApplication::translate("MainWindow", "Standard", nullptr));
        mToolBarView->setWindowTitle(QCoreApplication::translate("MainWindow", "Categories", nullptr));
        mToolBarFilter->setWindowTitle(QCoreApplication::translate("MainWindow", "Filter", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
