/********************************************************************************
** Form generated from reading UI file 'fileview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILEVIEW_H
#define UI_FILEVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_Fileview
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *mText;
    QDialogButtonBox *mButtons;

    void setupUi(QDialog *Fileview)
    {
        if (Fileview->objectName().isEmpty())
            Fileview->setObjectName("Fileview");
        Fileview->resize(400, 300);
        Fileview->setWindowTitle(QString::fromUtf8("Fileview"));
        verticalLayout = new QVBoxLayout(Fileview);
        verticalLayout->setObjectName("verticalLayout");
        mText = new QTextEdit(Fileview);
        mText->setObjectName("mText");
        mText->setReadOnly(true);

        verticalLayout->addWidget(mText);

        mButtons = new QDialogButtonBox(Fileview);
        mButtons->setObjectName("mButtons");
        mButtons->setOrientation(Qt::Horizontal);
        mButtons->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(mButtons);


        retranslateUi(Fileview);
        QObject::connect(mButtons, &QDialogButtonBox::accepted, Fileview, qOverload<>(&QDialog::accept));
        QObject::connect(mButtons, &QDialogButtonBox::rejected, Fileview, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(Fileview);
    } // setupUi

    void retranslateUi(QDialog *Fileview)
    {
        (void)Fileview;
    } // retranslateUi

};

namespace Ui {
    class Fileview: public Ui_Fileview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILEVIEW_H
