/********************************************************************************
** Form generated from reading UI file 'about.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUT_H
#define UI_ABOUT_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_About
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *mIcon;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *mVersion;
    QLabel *mName;
    QLabel *mCopyright;
    QLabel *mLicense;
    QLabel *mHomepage;
    QLabel *mUsedLibs;
    QDialogButtonBox *mButtons;

    void setupUi(QDialog *About)
    {
        if (About->objectName().isEmpty())
            About->setObjectName("About");
        About->resize(478, 375);
        verticalLayout_3 = new QVBoxLayout(About);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        mIcon = new QLabel(About);
        mIcon->setObjectName("mIcon");
        mIcon->setPixmap(QPixmap(QString::fromUtf8(":/cppcheck-gui.png")));

        verticalLayout_2->addWidget(mIcon);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        verticalLayout->addItem(horizontalSpacer);

        mVersion = new QLabel(About);
        mVersion->setObjectName("mVersion");

        verticalLayout->addWidget(mVersion);

        mName = new QLabel(About);
        mName->setObjectName("mName");
        mName->setWordWrap(true);

        verticalLayout->addWidget(mName);

        mCopyright = new QLabel(About);
        mCopyright->setObjectName("mCopyright");
        mCopyright->setWordWrap(true);

        verticalLayout->addWidget(mCopyright);

        mLicense = new QLabel(About);
        mLicense->setObjectName("mLicense");
        mLicense->setWordWrap(true);

        verticalLayout->addWidget(mLicense);

        mHomepage = new QLabel(About);
        mHomepage->setObjectName("mHomepage");
        mHomepage->setWordWrap(true);
        mHomepage->setOpenExternalLinks(true);

        verticalLayout->addWidget(mHomepage);

        mUsedLibs = new QLabel(About);
        mUsedLibs->setObjectName("mUsedLibs");

        verticalLayout->addWidget(mUsedLibs);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout);

        mButtons = new QDialogButtonBox(About);
        mButtons->setObjectName("mButtons");
        mButtons->setOrientation(Qt::Horizontal);
        mButtons->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(mButtons);


        retranslateUi(About);
        QObject::connect(mButtons, &QDialogButtonBox::accepted, About, qOverload<>(&QDialog::accept));
        QObject::connect(mButtons, &QDialogButtonBox::rejected, About, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(About);
    } // setupUi

    void retranslateUi(QDialog *About)
    {
        About->setWindowTitle(QCoreApplication::translate("About", "About Cppcheck", nullptr));
        mIcon->setText(QString());
        mVersion->setText(QCoreApplication::translate("About", "Version %1", nullptr));
        mName->setText(QCoreApplication::translate("About", "Cppcheck - A tool for static C/C++ code analysis.", nullptr));
        mCopyright->setText(QCoreApplication::translate("About", "Copyright \302\251 2007-%1 Cppcheck team.", nullptr));
        mLicense->setText(QCoreApplication::translate("About", "This program is licensed under the terms\n"
"of the GNU General Public License version 3", nullptr));
        mHomepage->setText(QCoreApplication::translate("About", "Visit Cppcheck homepage at %1", nullptr));
        mUsedLibs->setText(QCoreApplication::translate("About", "<html><head/><body><p>Many thanks to these libraries that we use:</p><ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">PCRE</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">PicoJSON</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Qt</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">TinyXML2</li><li style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Boost</li></ul></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class About: public Ui_About {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUT_H
