/********************************************************************************
** Form generated from reading UI file 'threaddetails.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_THREADDETAILS_H
#define UI_THREADDETAILS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ThreadDetails
{
public:
    QVBoxLayout *verticalLayout;
    QPlainTextEdit *plainTextEdit;

    void setupUi(QWidget *ThreadDetails)
    {
        if (ThreadDetails->objectName().isEmpty())
            ThreadDetails->setObjectName("ThreadDetails");
        ThreadDetails->resize(640, 300);
        verticalLayout = new QVBoxLayout(ThreadDetails);
        verticalLayout->setObjectName("verticalLayout");
        plainTextEdit = new QPlainTextEdit(ThreadDetails);
        plainTextEdit->setObjectName("plainTextEdit");
        plainTextEdit->setReadOnly(true);

        verticalLayout->addWidget(plainTextEdit);


        retranslateUi(ThreadDetails);

        QMetaObject::connectSlotsByName(ThreadDetails);
    } // setupUi

    void retranslateUi(QWidget *ThreadDetails)
    {
        ThreadDetails->setWindowTitle(QCoreApplication::translate("ThreadDetails", "Thread Details", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ThreadDetails: public Ui_ThreadDetails {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_THREADDETAILS_H
