/********************************************************************************
** Form generated from reading UI file 'newsuppressiondialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWSUPPRESSIONDIALOG_H
#define UI_NEWSUPPRESSIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_NewSuppressionDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *label_2;
    QLineEdit *mTextFileName;
    QLabel *label_3;
    QLineEdit *mTextLineNumber;
    QLabel *label_4;
    QLineEdit *mTextSymbolName;
    QComboBox *mComboErrorId;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *NewSuppressionDialog)
    {
        if (NewSuppressionDialog->objectName().isEmpty())
            NewSuppressionDialog->setObjectName("NewSuppressionDialog");
        NewSuppressionDialog->setWindowModality(Qt::ApplicationModal);
        NewSuppressionDialog->resize(334, 184);
        verticalLayout = new QVBoxLayout(NewSuppressionDialog);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(NewSuppressionDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(NewSuppressionDialog);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        mTextFileName = new QLineEdit(NewSuppressionDialog);
        mTextFileName->setObjectName("mTextFileName");

        gridLayout->addWidget(mTextFileName, 1, 1, 1, 1);

        label_3 = new QLabel(NewSuppressionDialog);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        mTextLineNumber = new QLineEdit(NewSuppressionDialog);
        mTextLineNumber->setObjectName("mTextLineNumber");

        gridLayout->addWidget(mTextLineNumber, 2, 1, 1, 1);

        label_4 = new QLabel(NewSuppressionDialog);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        mTextSymbolName = new QLineEdit(NewSuppressionDialog);
        mTextSymbolName->setObjectName("mTextSymbolName");

        gridLayout->addWidget(mTextSymbolName, 3, 1, 1, 1);

        mComboErrorId = new QComboBox(NewSuppressionDialog);
        mComboErrorId->setObjectName("mComboErrorId");
        mComboErrorId->setEditable(true);

        gridLayout->addWidget(mComboErrorId, 0, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        buttonBox = new QDialogButtonBox(NewSuppressionDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(mComboErrorId, mTextFileName);
        QWidget::setTabOrder(mTextFileName, mTextLineNumber);
        QWidget::setTabOrder(mTextLineNumber, mTextSymbolName);

        retranslateUi(NewSuppressionDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, NewSuppressionDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, NewSuppressionDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(NewSuppressionDialog);
    } // setupUi

    void retranslateUi(QDialog *NewSuppressionDialog)
    {
        NewSuppressionDialog->setWindowTitle(QCoreApplication::translate("NewSuppressionDialog", "New suppression", nullptr));
        label->setText(QCoreApplication::translate("NewSuppressionDialog", "Error ID", nullptr));
        label_2->setText(QCoreApplication::translate("NewSuppressionDialog", "File name", nullptr));
        label_3->setText(QCoreApplication::translate("NewSuppressionDialog", "Line number", nullptr));
        label_4->setText(QCoreApplication::translate("NewSuppressionDialog", "Symbol name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NewSuppressionDialog: public Ui_NewSuppressionDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NEWSUPPRESSIONDIALOG_H
