/********************************************************************************
** Form generated from reading UI file 'scratchpad.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRATCHPAD_H
#define UI_SCRATCHPAD_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "codeeditor.h"

QT_BEGIN_NAMESPACE

class Ui_ScratchPad
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_sourcecode;
    CodeEditor *plainTextEdit;
    QLabel *label_filename;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QLineEdit *lineEdit;
    QPushButton *mCheckButton;
    QDialogButtonBox *mButtonBox;

    void setupUi(QDialog *ScratchPad)
    {
        if (ScratchPad->objectName().isEmpty())
            ScratchPad->setObjectName("ScratchPad");
        ScratchPad->resize(574, 600);
        verticalLayout = new QVBoxLayout(ScratchPad);
        verticalLayout->setObjectName("verticalLayout");
        label_sourcecode = new QLabel(ScratchPad);
        label_sourcecode->setObjectName("label_sourcecode");

        verticalLayout->addWidget(label_sourcecode);

        plainTextEdit = new CodeEditor(ScratchPad);
        plainTextEdit->setObjectName("plainTextEdit");
        QFont font;
        font.setFamilies({QString::fromUtf8("Courier New")});
        font.setPointSize(10);
        plainTextEdit->setFont(font);

        verticalLayout->addWidget(plainTextEdit);

        label_filename = new QLabel(ScratchPad);
        label_filename->setObjectName("label_filename");

        verticalLayout->addWidget(label_filename);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        lineEdit = new QLineEdit(ScratchPad);
        lineEdit->setObjectName("lineEdit");

        horizontalLayout->addWidget(lineEdit);

        mCheckButton = new QPushButton(ScratchPad);
        mCheckButton->setObjectName("mCheckButton");

        horizontalLayout->addWidget(mCheckButton);

        mButtonBox = new QDialogButtonBox(ScratchPad);
        mButtonBox->setObjectName("mButtonBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mButtonBox->sizePolicy().hasHeightForWidth());
        mButtonBox->setSizePolicy(sizePolicy);
        mButtonBox->setOrientation(Qt::Horizontal);
        mButtonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(mButtonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(ScratchPad);
        QObject::connect(mButtonBox, &QDialogButtonBox::rejected, ScratchPad, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ScratchPad);
    } // setupUi

    void retranslateUi(QDialog *ScratchPad)
    {
        ScratchPad->setWindowTitle(QCoreApplication::translate("ScratchPad", "Scratchpad", nullptr));
        label_sourcecode->setText(QCoreApplication::translate("ScratchPad", "Copy or write some C/C++ code here:", nullptr));
        label_filename->setText(QCoreApplication::translate("ScratchPad", "Optionally enter a filename (mainly for automatic language detection) and click on \"Check\":", nullptr));
        lineEdit->setPlaceholderText(QCoreApplication::translate("ScratchPad", "filename", nullptr));
        mCheckButton->setText(QCoreApplication::translate("ScratchPad", "Check", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ScratchPad: public Ui_ScratchPad {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCRATCHPAD_H
