/********************************************************************************
** Form generated from reading UI file 'applicationdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_APPLICATIONDIALOG_H
#define UI_APPLICATIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ApplicationDialog
{
public:
    QVBoxLayout *verticalLayout_4;
    QLabel *label;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *label_2;
    QLabel *label_3;
    QLabel *label_4;
    QVBoxLayout *verticalLayout;
    QLineEdit *mName;
    QLineEdit *mPath;
    QLineEdit *mParameters;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *mButtonBrowse;
    QDialogButtonBox *mButtons;

    void setupUi(QDialog *ApplicationDialog)
    {
        if (ApplicationDialog->objectName().isEmpty())
            ApplicationDialog->setObjectName("ApplicationDialog");
        ApplicationDialog->setWindowModality(Qt::WindowModal);
        ApplicationDialog->resize(569, 471);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ApplicationDialog->sizePolicy().hasHeightForWidth());
        ApplicationDialog->setSizePolicy(sizePolicy);
        verticalLayout_4 = new QVBoxLayout(ApplicationDialog);
        verticalLayout_4->setObjectName("verticalLayout_4");
        label = new QLabel(ApplicationDialog);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setMinimumSize(QSize(0, 0));
        label->setTextFormat(Qt::AutoText);
        label->setScaledContents(false);
        label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label->setWordWrap(true);

        verticalLayout_4->addWidget(label);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        label_2 = new QLabel(ApplicationDialog);
        label_2->setObjectName("label_2");

        verticalLayout_2->addWidget(label_2);

        label_3 = new QLabel(ApplicationDialog);
        label_3->setObjectName("label_3");

        verticalLayout_2->addWidget(label_3);

        label_4 = new QLabel(ApplicationDialog);
        label_4->setObjectName("label_4");

        verticalLayout_2->addWidget(label_4);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        mName = new QLineEdit(ApplicationDialog);
        mName->setObjectName("mName");

        verticalLayout->addWidget(mName);

        mPath = new QLineEdit(ApplicationDialog);
        mPath->setObjectName("mPath");

        verticalLayout->addWidget(mPath);

        mParameters = new QLineEdit(ApplicationDialog);
        mParameters->setObjectName("mParameters");

        verticalLayout->addWidget(mParameters);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        mButtonBrowse = new QPushButton(ApplicationDialog);
        mButtonBrowse->setObjectName("mButtonBrowse");

        horizontalLayout_2->addWidget(mButtonBrowse);


        verticalLayout_3->addLayout(horizontalLayout_2);


        verticalLayout_4->addLayout(verticalLayout_3);

        mButtons = new QDialogButtonBox(ApplicationDialog);
        mButtons->setObjectName("mButtons");
        mButtons->setOrientation(Qt::Horizontal);
        mButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_4->addWidget(mButtons);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(mName);
        label_3->setBuddy(mPath);
        label_4->setBuddy(mParameters);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(mName, mPath);
        QWidget::setTabOrder(mPath, mParameters);
        QWidget::setTabOrder(mParameters, mButtonBrowse);
        QWidget::setTabOrder(mButtonBrowse, mButtons);

        retranslateUi(ApplicationDialog);
        QObject::connect(mButtons, &QDialogButtonBox::accepted, ApplicationDialog, qOverload<>(&QDialog::accept));
        QObject::connect(mButtons, &QDialogButtonBox::rejected, ApplicationDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ApplicationDialog);
    } // setupUi

    void retranslateUi(QDialog *ApplicationDialog)
    {
        ApplicationDialog->setWindowTitle(QCoreApplication::translate("ApplicationDialog", "Add an application", nullptr));
        label->setText(QCoreApplication::translate("ApplicationDialog", "Here you can add an application that can open error files. Specify a name for the application, the application executable and command line parameters for the application.\n"
"\n"
"The following texts in parameters are replaced with appropriate values when application is executed:\n"
"(file) - Filename containing the error\n"
"(line) - Line number containing the error\n"
"(message) - Error message\n"
"(severity) - Error severity\n"
"\n"
"Example opening a file with Kate and make Kate scroll to the correct line:\n"
"Executable: kate\n"
"Parameters: -l(line) (file)", nullptr));
        label_2->setText(QCoreApplication::translate("ApplicationDialog", "&Name:", nullptr));
        label_3->setText(QCoreApplication::translate("ApplicationDialog", "&Executable:", nullptr));
        label_4->setText(QCoreApplication::translate("ApplicationDialog", "&Parameters:", nullptr));
        mButtonBrowse->setText(QCoreApplication::translate("ApplicationDialog", "Browse", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ApplicationDialog: public Ui_ApplicationDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_APPLICATIONDIALOG_H
