/****************************************************************************
** Meta object code from reading C++ file 'codeeditstyledialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../gui/codeeditstyledialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'codeeditstyledialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15StyleEditDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto StyleEditDialog::qt_create_metaobjectdata<qt_meta_tag_ZN15StyleEditDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StyleEditDialog",
        "resetStyle",
        "",
        "setStyleDefaultLight",
        "setStyleDefaultDark",
        "colorChangedWidgetFG",
        "QColor",
        "newColor",
        "colorChangedWidgetBG",
        "colorChangedHighlightBG",
        "colorChangedLineNumFG",
        "colorChangedLineNumBG",
        "colorChangedKeywordFG",
        "weightChangedKeyword",
        "QFont::Weight",
        "newWeight",
        "colorChangedClassFG",
        "weightChangedClass",
        "colorChangedQuoteFG",
        "weightChangedQuote",
        "colorChangedCommentFG",
        "weightChangedComment",
        "colorChangedSymbolFG",
        "colorChangedSymbolBG",
        "weightChangedSymbol"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'resetStyle'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setStyleDefaultLight'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setStyleDefaultDark'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'colorChangedWidgetFG'
        QtMocHelpers::SlotData<void(const QColor &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'colorChangedWidgetBG'
        QtMocHelpers::SlotData<void(const QColor &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'colorChangedHighlightBG'
        QtMocHelpers::SlotData<void(const QColor &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'colorChangedLineNumFG'
        QtMocHelpers::SlotData<void(const QColor &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'colorChangedLineNumBG'
        QtMocHelpers::SlotData<void(const QColor &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'colorChangedKeywordFG'
        QtMocHelpers::SlotData<void(const QColor &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'weightChangedKeyword'
        QtMocHelpers::SlotData<void(QFont::Weight)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'colorChangedClassFG'
        QtMocHelpers::SlotData<void(const QColor &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'weightChangedClass'
        QtMocHelpers::SlotData<void(QFont::Weight)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'colorChangedQuoteFG'
        QtMocHelpers::SlotData<void(const QColor &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'weightChangedQuote'
        QtMocHelpers::SlotData<void(QFont::Weight)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'colorChangedCommentFG'
        QtMocHelpers::SlotData<void(const QColor &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'weightChangedComment'
        QtMocHelpers::SlotData<void(QFont::Weight)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'colorChangedSymbolFG'
        QtMocHelpers::SlotData<void(const QColor &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'colorChangedSymbolBG'
        QtMocHelpers::SlotData<void(const QColor &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'weightChangedSymbol'
        QtMocHelpers::SlotData<void(QFont::Weight)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StyleEditDialog, qt_meta_tag_ZN15StyleEditDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StyleEditDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StyleEditDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StyleEditDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15StyleEditDialogE_t>.metaTypes,
    nullptr
} };

void StyleEditDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StyleEditDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resetStyle(); break;
        case 1: _t->setStyleDefaultLight(); break;
        case 2: _t->setStyleDefaultDark(); break;
        case 3: _t->colorChangedWidgetFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->colorChangedWidgetBG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->colorChangedHighlightBG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->colorChangedLineNumFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 7: _t->colorChangedLineNumBG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 8: _t->colorChangedKeywordFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 9: _t->weightChangedKeyword((*reinterpret_cast<std::add_pointer_t<QFont::Weight>>(_a[1]))); break;
        case 10: _t->colorChangedClassFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 11: _t->weightChangedClass((*reinterpret_cast<std::add_pointer_t<QFont::Weight>>(_a[1]))); break;
        case 12: _t->colorChangedQuoteFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 13: _t->weightChangedQuote((*reinterpret_cast<std::add_pointer_t<QFont::Weight>>(_a[1]))); break;
        case 14: _t->colorChangedCommentFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 15: _t->weightChangedComment((*reinterpret_cast<std::add_pointer_t<QFont::Weight>>(_a[1]))); break;
        case 16: _t->colorChangedSymbolFG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 17: _t->colorChangedSymbolBG((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 18: _t->weightChangedSymbol((*reinterpret_cast<std::add_pointer_t<QFont::Weight>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *StyleEditDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StyleEditDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int StyleEditDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
