/********************************************************************************
** Form generated from reading UI file 'settings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGS_H
#define UI_SETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Settings
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QLineEdit *mJobs;
    QLabel *label_3;
    QLabel *mLblMaxThreads;
    QSpacerItem *horizontalSpacer;
    QCheckBox *mForce;
    QCheckBox *mShowFullPath;
    QCheckBox *mShowNoErrorsMessage;
    QCheckBox *mShowErrorId;
    QCheckBox *mInlineSuppressions;
    QCheckBox *mEnableInconclusive;
    QCheckBox *mShowStatistics;
    QCheckBox *mCheckForUpdates;
    QSpacerItem *verticalSpacer_2;
    QCheckBox *mShowDebugWarnings;
    QWidget *tab_2;
    QHBoxLayout *horizontalLayout_3;
    QListWidget *mListWidget;
    QVBoxLayout *verticalLayout_2;
    QPushButton *mBtnAddApplication;
    QPushButton *mBtnEditApplication;
    QPushButton *mBtnRemoveApplication;
    QPushButton *mBtnDefaultApplication;
    QSpacerItem *verticalSpacer_4;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *mSaveAllErrors;
    QCheckBox *mSaveFullPath;
    QSpacerItem *verticalSpacer;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_5;
    QListWidget *mListLanguages;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_13;
    QHBoxLayout *horizontalLayout;
    QLineEdit *mEditPythonPath;
    QPushButton *mBtnBrowsePythonPath;
    QLabel *mPythonPathWarning;
    QGroupBox *mGroupBoxMisra;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_4;
    QLineEdit *mEditMisraFile;
    QPushButton *mBtnBrowseMisraFile;
    QSpacerItem *verticalSpacer_3;
    QWidget *mTabClang;
    QVBoxLayout *verticalLayout_8;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_5;
    QLineEdit *mEditClangPath;
    QPushButton *mBtnBrowseClangPath;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_9;
    QLabel *mLabelVsIncludePaths;
    QLineEdit *mEditVsIncludePaths;
    QSpacerItem *verticalSpacer_6;
    QWidget *mTabEditorStyle;
    QVBoxLayout *verticalLayout_11;
    QGroupBox *gboxEditorStyle;
    QVBoxLayout *verticalLayout_12;
    QRadioButton *mThemeSystem;
    QRadioButton *mThemeLight;
    QRadioButton *mThemeDark;
    QHBoxLayout *hlEditCustom;
    QRadioButton *mThemeCustom;
    QPushButton *mBtnEditTheme;
    QSpacerItem *hspaceCustom;
    QSpacerItem *vspaceEditorStyle;
    QDialogButtonBox *mButtons;

    void setupUi(QDialog *Settings)
    {
        if (Settings->objectName().isEmpty())
            Settings->setObjectName("Settings");
        Settings->resize(627, 411);
        verticalLayout = new QVBoxLayout(Settings);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(Settings);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setSizeConstraint(QLayout::SetDefaultConstraint);
        label = new QLabel(tab);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(label);

        mJobs = new QLineEdit(tab);
        mJobs->setObjectName("mJobs");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mJobs->sizePolicy().hasHeightForWidth());
        mJobs->setSizePolicy(sizePolicy1);
        mJobs->setMinimumSize(QSize(100, 20));
        mJobs->setBaseSize(QSize(100, 20));
        mJobs->setText(QString::fromUtf8(""));
        mJobs->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(mJobs);

        label_3 = new QLabel(tab);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        mLblMaxThreads = new QLabel(tab);
        mLblMaxThreads->setObjectName("mLblMaxThreads");
        mLblMaxThreads->setText(QString::fromUtf8("TextLabel"));

        horizontalLayout_2->addWidget(mLblMaxThreads);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_4->addLayout(horizontalLayout_2);

        mForce = new QCheckBox(tab);
        mForce->setObjectName("mForce");

        verticalLayout_4->addWidget(mForce);

        mShowFullPath = new QCheckBox(tab);
        mShowFullPath->setObjectName("mShowFullPath");

        verticalLayout_4->addWidget(mShowFullPath);

        mShowNoErrorsMessage = new QCheckBox(tab);
        mShowNoErrorsMessage->setObjectName("mShowNoErrorsMessage");

        verticalLayout_4->addWidget(mShowNoErrorsMessage);

        mShowErrorId = new QCheckBox(tab);
        mShowErrorId->setObjectName("mShowErrorId");

        verticalLayout_4->addWidget(mShowErrorId);

        mInlineSuppressions = new QCheckBox(tab);
        mInlineSuppressions->setObjectName("mInlineSuppressions");

        verticalLayout_4->addWidget(mInlineSuppressions);

        mEnableInconclusive = new QCheckBox(tab);
        mEnableInconclusive->setObjectName("mEnableInconclusive");

        verticalLayout_4->addWidget(mEnableInconclusive);

        mShowStatistics = new QCheckBox(tab);
        mShowStatistics->setObjectName("mShowStatistics");

        verticalLayout_4->addWidget(mShowStatistics);

        mCheckForUpdates = new QCheckBox(tab);
        mCheckForUpdates->setObjectName("mCheckForUpdates");

        verticalLayout_4->addWidget(mCheckForUpdates);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        mShowDebugWarnings = new QCheckBox(tab);
        mShowDebugWarnings->setObjectName("mShowDebugWarnings");

        verticalLayout_4->addWidget(mShowDebugWarnings);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        horizontalLayout_3 = new QHBoxLayout(tab_2);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        mListWidget = new QListWidget(tab_2);
        mListWidget->setObjectName("mListWidget");

        horizontalLayout_3->addWidget(mListWidget);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        mBtnAddApplication = new QPushButton(tab_2);
        mBtnAddApplication->setObjectName("mBtnAddApplication");

        verticalLayout_2->addWidget(mBtnAddApplication);

        mBtnEditApplication = new QPushButton(tab_2);
        mBtnEditApplication->setObjectName("mBtnEditApplication");

        verticalLayout_2->addWidget(mBtnEditApplication);

        mBtnRemoveApplication = new QPushButton(tab_2);
        mBtnRemoveApplication->setObjectName("mBtnRemoveApplication");

        verticalLayout_2->addWidget(mBtnRemoveApplication);

        mBtnDefaultApplication = new QPushButton(tab_2);
        mBtnDefaultApplication->setObjectName("mBtnDefaultApplication");

        verticalLayout_2->addWidget(mBtnDefaultApplication);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_4);


        horizontalLayout_3->addLayout(verticalLayout_2);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_3 = new QVBoxLayout(tab_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        mSaveAllErrors = new QCheckBox(tab_3);
        mSaveAllErrors->setObjectName("mSaveAllErrors");

        verticalLayout_3->addWidget(mSaveAllErrors);

        mSaveFullPath = new QCheckBox(tab_3);
        mSaveFullPath->setObjectName("mSaveFullPath");

        verticalLayout_3->addWidget(mSaveFullPath);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout_5 = new QVBoxLayout(tab_4);
        verticalLayout_5->setObjectName("verticalLayout_5");
        mListLanguages = new QListWidget(tab_4);
        mListLanguages->setObjectName("mListLanguages");
        mListLanguages->setSelectionBehavior(QAbstractItemView::SelectRows);

        verticalLayout_5->addWidget(mListLanguages);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName("tab_5");
        verticalLayout_6 = new QVBoxLayout(tab_5);
        verticalLayout_6->setObjectName("verticalLayout_6");
        groupBox_3 = new QGroupBox(tab_5);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_13 = new QVBoxLayout(groupBox_3);
        verticalLayout_13->setObjectName("verticalLayout_13");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mEditPythonPath = new QLineEdit(groupBox_3);
        mEditPythonPath->setObjectName("mEditPythonPath");

        horizontalLayout->addWidget(mEditPythonPath);

        mBtnBrowsePythonPath = new QPushButton(groupBox_3);
        mBtnBrowsePythonPath->setObjectName("mBtnBrowsePythonPath");

        horizontalLayout->addWidget(mBtnBrowsePythonPath);


        verticalLayout_13->addLayout(horizontalLayout);

        mPythonPathWarning = new QLabel(groupBox_3);
        mPythonPathWarning->setObjectName("mPythonPathWarning");

        verticalLayout_13->addWidget(mPythonPathWarning);


        verticalLayout_6->addWidget(groupBox_3);

        mGroupBoxMisra = new QGroupBox(tab_5);
        mGroupBoxMisra->setObjectName("mGroupBoxMisra");
        verticalLayout_7 = new QVBoxLayout(mGroupBoxMisra);
        verticalLayout_7->setObjectName("verticalLayout_7");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_4 = new QLabel(mGroupBoxMisra);
        label_4->setObjectName("label_4");

        horizontalLayout_4->addWidget(label_4);

        mEditMisraFile = new QLineEdit(mGroupBoxMisra);
        mEditMisraFile->setObjectName("mEditMisraFile");

        horizontalLayout_4->addWidget(mEditMisraFile);

        mBtnBrowseMisraFile = new QPushButton(mGroupBoxMisra);
        mBtnBrowseMisraFile->setObjectName("mBtnBrowseMisraFile");

        horizontalLayout_4->addWidget(mBtnBrowseMisraFile);


        verticalLayout_7->addLayout(horizontalLayout_4);


        verticalLayout_6->addWidget(mGroupBoxMisra);

        verticalSpacer_3 = new QSpacerItem(20, 168, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_3);

        tabWidget->addTab(tab_5, QString());
        mTabClang = new QWidget();
        mTabClang->setObjectName("mTabClang");
        verticalLayout_8 = new QVBoxLayout(mTabClang);
        verticalLayout_8->setObjectName("verticalLayout_8");
        groupBox = new QGroupBox(mTabClang);
        groupBox->setObjectName("groupBox");
        horizontalLayout_5 = new QHBoxLayout(groupBox);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        mEditClangPath = new QLineEdit(groupBox);
        mEditClangPath->setObjectName("mEditClangPath");
        mEditClangPath->setReadOnly(false);

        horizontalLayout_5->addWidget(mEditClangPath);

        mBtnBrowseClangPath = new QPushButton(groupBox);
        mBtnBrowseClangPath->setObjectName("mBtnBrowseClangPath");

        horizontalLayout_5->addWidget(mBtnBrowseClangPath);


        verticalLayout_8->addWidget(groupBox);

        groupBox_2 = new QGroupBox(mTabClang);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_9 = new QVBoxLayout(groupBox_2);
        verticalLayout_9->setObjectName("verticalLayout_9");
        mLabelVsIncludePaths = new QLabel(groupBox_2);
        mLabelVsIncludePaths->setObjectName("mLabelVsIncludePaths");
        mLabelVsIncludePaths->setWordWrap(true);

        verticalLayout_9->addWidget(mLabelVsIncludePaths);

        mEditVsIncludePaths = new QLineEdit(groupBox_2);
        mEditVsIncludePaths->setObjectName("mEditVsIncludePaths");

        verticalLayout_9->addWidget(mEditVsIncludePaths);


        verticalLayout_8->addWidget(groupBox_2);

        verticalSpacer_6 = new QSpacerItem(20, 50, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_6);

        tabWidget->addTab(mTabClang, QString());
        mTabEditorStyle = new QWidget();
        mTabEditorStyle->setObjectName("mTabEditorStyle");
        verticalLayout_11 = new QVBoxLayout(mTabEditorStyle);
        verticalLayout_11->setObjectName("verticalLayout_11");
        gboxEditorStyle = new QGroupBox(mTabEditorStyle);
        gboxEditorStyle->setObjectName("gboxEditorStyle");
        verticalLayout_12 = new QVBoxLayout(gboxEditorStyle);
        verticalLayout_12->setObjectName("verticalLayout_12");
        mThemeSystem = new QRadioButton(gboxEditorStyle);
        mThemeSystem->setObjectName("mThemeSystem");

        verticalLayout_12->addWidget(mThemeSystem);

        mThemeLight = new QRadioButton(gboxEditorStyle);
        mThemeLight->setObjectName("mThemeLight");

        verticalLayout_12->addWidget(mThemeLight);

        mThemeDark = new QRadioButton(gboxEditorStyle);
        mThemeDark->setObjectName("mThemeDark");

        verticalLayout_12->addWidget(mThemeDark);

        hlEditCustom = new QHBoxLayout();
        hlEditCustom->setObjectName("hlEditCustom");
        mThemeCustom = new QRadioButton(gboxEditorStyle);
        mThemeCustom->setObjectName("mThemeCustom");

        hlEditCustom->addWidget(mThemeCustom);

        mBtnEditTheme = new QPushButton(gboxEditorStyle);
        mBtnEditTheme->setObjectName("mBtnEditTheme");

        hlEditCustom->addWidget(mBtnEditTheme);

        hspaceCustom = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hlEditCustom->addItem(hspaceCustom);


        verticalLayout_12->addLayout(hlEditCustom);

        vspaceEditorStyle = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_12->addItem(vspaceEditorStyle);


        verticalLayout_11->addWidget(gboxEditorStyle);

        tabWidget->addTab(mTabEditorStyle, QString());

        verticalLayout->addWidget(tabWidget);

        mButtons = new QDialogButtonBox(Settings);
        mButtons->setObjectName("mButtons");
        mButtons->setOrientation(Qt::Horizontal);
        mButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(mButtons);

#if QT_CONFIG(shortcut)
        label->setBuddy(mJobs);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabWidget, mJobs);
        QWidget::setTabOrder(mJobs, mForce);
        QWidget::setTabOrder(mForce, mShowFullPath);
        QWidget::setTabOrder(mShowFullPath, mShowNoErrorsMessage);
        QWidget::setTabOrder(mShowNoErrorsMessage, mInlineSuppressions);
        QWidget::setTabOrder(mInlineSuppressions, mListWidget);
        QWidget::setTabOrder(mListWidget, mBtnAddApplication);
        QWidget::setTabOrder(mBtnAddApplication, mBtnEditApplication);
        QWidget::setTabOrder(mBtnEditApplication, mBtnRemoveApplication);
        QWidget::setTabOrder(mBtnRemoveApplication, mBtnDefaultApplication);
        QWidget::setTabOrder(mBtnDefaultApplication, mSaveAllErrors);
        QWidget::setTabOrder(mSaveAllErrors, mSaveFullPath);
        QWidget::setTabOrder(mSaveFullPath, mListLanguages);
        QWidget::setTabOrder(mListLanguages, mEnableInconclusive);
        QWidget::setTabOrder(mEnableInconclusive, mShowStatistics);
        QWidget::setTabOrder(mShowStatistics, mShowDebugWarnings);
        QWidget::setTabOrder(mShowDebugWarnings, mButtons);

        retranslateUi(Settings);
        QObject::connect(mButtons, &QDialogButtonBox::accepted, Settings, qOverload<>(&QDialog::accept));
        QObject::connect(mButtons, &QDialogButtonBox::rejected, Settings, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Settings);
    } // setupUi

    void retranslateUi(QDialog *Settings)
    {
        Settings->setWindowTitle(QCoreApplication::translate("Settings", "Preferences", nullptr));
        label->setText(QCoreApplication::translate("Settings", "Number of threads: ", nullptr));
        label_3->setText(QCoreApplication::translate("Settings", "Max count:", nullptr));
        mForce->setText(QCoreApplication::translate("Settings", "Force checking all #ifdef configurations", nullptr));
        mShowFullPath->setText(QCoreApplication::translate("Settings", "Show full path of files", nullptr));
        mShowNoErrorsMessage->setText(QCoreApplication::translate("Settings", "Show \"No errors found\" message when no errors found", nullptr));
        mShowErrorId->setText(QCoreApplication::translate("Settings", "Display error Id in column \"Id\"", nullptr));
        mInlineSuppressions->setText(QCoreApplication::translate("Settings", "Enable inline suppressions", nullptr));
        mEnableInconclusive->setText(QCoreApplication::translate("Settings", "Check for inconclusive errors also", nullptr));
        mShowStatistics->setText(QCoreApplication::translate("Settings", "Show statistics on check completion", nullptr));
        mCheckForUpdates->setText(QCoreApplication::translate("Settings", "Check for updates", nullptr));
        mShowDebugWarnings->setText(QCoreApplication::translate("Settings", "Show internal warnings in log", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("Settings", "General", nullptr));
        mBtnAddApplication->setText(QCoreApplication::translate("Settings", "Add...", nullptr));
        mBtnEditApplication->setText(QCoreApplication::translate("Settings", "Edit...", nullptr));
        mBtnRemoveApplication->setText(QCoreApplication::translate("Settings", "Remove", nullptr));
        mBtnDefaultApplication->setText(QCoreApplication::translate("Settings", "Set as default", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("Settings", "Applications", nullptr));
        mSaveAllErrors->setText(QCoreApplication::translate("Settings", "Save all errors when creating report", nullptr));
        mSaveFullPath->setText(QCoreApplication::translate("Settings", "Save full path to files in reports", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("Settings", "Reports", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("Settings", "Language", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("Settings", "Python binary (leave this empty to use python in the PATH)", nullptr));
        mBtnBrowsePythonPath->setText(QCoreApplication::translate("Settings", "...", nullptr));
        mPythonPathWarning->setText(QString());
        mGroupBoxMisra->setTitle(QCoreApplication::translate("Settings", "MISRA addon", nullptr));
        label_4->setText(QCoreApplication::translate("Settings", "MISRA rule texts file", nullptr));
#if QT_CONFIG(tooltip)
        mEditMisraFile->setToolTip(QCoreApplication::translate("Settings", "<html><head/><body><p>Copy/paste the text from Appendix A &quot;Summary of guidelines&quot; from the MISRA C 2012 pdf to a text file.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        mBtnBrowseMisraFile->setText(QCoreApplication::translate("Settings", "...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("Settings", "Addons", nullptr));
        groupBox->setTitle(QCoreApplication::translate("Settings", "Clang path (leave empty to use system PATH)", nullptr));
        mBtnBrowseClangPath->setText(QCoreApplication::translate("Settings", "...", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("Settings", "Visual Studio headers", nullptr));
        mLabelVsIncludePaths->setText(QCoreApplication::translate("Settings", "<html><head/><body><p>Paths to Visual Studio headers, separated by semicolon ';'.</p><p>You can open a Visual Studio command prompt, write &quot;SET INCLUDE&quot;. Then copy/paste the paths.</p></body></html>", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabClang), QCoreApplication::translate("Settings", "Clang", nullptr));
        gboxEditorStyle->setTitle(QCoreApplication::translate("Settings", "Code Editor Style", nullptr));
        mThemeSystem->setText(QCoreApplication::translate("Settings", "System Style", nullptr));
        mThemeLight->setText(QCoreApplication::translate("Settings", "Default Light Style", nullptr));
        mThemeDark->setText(QCoreApplication::translate("Settings", "Default Dark Style", nullptr));
        mThemeCustom->setText(QCoreApplication::translate("Settings", "Custom", nullptr));
        mBtnEditTheme->setText(QCoreApplication::translate("Settings", "Edit...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(mTabEditorStyle), QCoreApplication::translate("Settings", "Code Editor", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Settings: public Ui_Settings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGS_H
