/****************************************************************************
** Meta object code from reading C++ file 'resultsview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../gui/resultsview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'resultsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11ResultsViewE_t {};
} // unnamed namespace

template <> constexpr inline auto ResultsView::qt_create_metaobjectdata<qt_meta_tag_ZN11ResultsViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ResultsView",
        "gotResults",
        "",
        "resultsHidden",
        "hidden",
        "checkSelected",
        "selectedFilesList",
        "suppressIds",
        "ids",
        "showResults",
        "ShowTypes::ShowType",
        "type",
        "show",
        "showCppcheckResults",
        "showClangResults",
        "collapseAllResults",
        "expandAllResults",
        "showHiddenResults",
        "filesCheckedProgress",
        "value",
        "description",
        "error",
        "ErrorItem",
        "item",
        "filterResults",
        "filter",
        "updateDetails",
        "const ResultItem*",
        "print",
        "QPrinter*",
        "printer",
        "printPreview",
        "log",
        "str",
        "debugError",
        "logClear",
        "logCopyEntry",
        "logCopyComplete",
        "on_mListLog_customContextMenuRequested",
        "QPoint",
        "pos"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'gotResults'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resultsHidden'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'checkSelected'
        QtMocHelpers::SignalData<void(QStringList)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 6 },
        }}),
        // Signal 'suppressIds'
        QtMocHelpers::SignalData<void(QStringList)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 8 },
        }}),
        // Signal 'showResults'
        QtMocHelpers::SignalData<void(ShowTypes::ShowType, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'showCppcheckResults'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'showClangResults'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'collapseAllResults'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expandAllResults'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showHiddenResults'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filesCheckedProgress'
        QtMocHelpers::SlotData<void(int, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::QString, 20 },
        }}),
        // Slot 'error'
        QtMocHelpers::SlotData<void(const ErrorItem &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'filterResults'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'updateDetails'
        QtMocHelpers::SlotData<void(const ResultItem *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 23 },
        }}),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'print'
        QtMocHelpers::SlotData<void(QPrinter *) const>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'printPreview'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'log'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'debugError'
        QtMocHelpers::SlotData<void(const ErrorItem &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'logClear'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'logCopyEntry'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'logCopyComplete'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_mListLog_customContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ResultsView, qt_meta_tag_ZN11ResultsViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ResultsView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ResultsViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ResultsViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11ResultsViewE_t>.metaTypes,
    nullptr
} };

void ResultsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ResultsView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->gotResults(); break;
        case 1: _t->resultsHidden((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->checkSelected((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->suppressIds((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->showResults((*reinterpret_cast<std::add_pointer_t<ShowTypes::ShowType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->showCppcheckResults((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->showClangResults((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->collapseAllResults(); break;
        case 8: _t->expandAllResults(); break;
        case 9: _t->showHiddenResults(); break;
        case 10: _t->filesCheckedProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->error((*reinterpret_cast<std::add_pointer_t<ErrorItem>>(_a[1]))); break;
        case 12: _t->filterResults((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->updateDetails((*reinterpret_cast<std::add_pointer_t<const ResultItem*>>(_a[1]))); break;
        case 14: _t->print(); break;
        case 15: _t->print((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 16: _t->printPreview(); break;
        case 17: _t->log((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->debugError((*reinterpret_cast<std::add_pointer_t<ErrorItem>>(_a[1]))); break;
        case 19: _t->logClear(); break;
        case 20: _t->logCopyEntry(); break;
        case 21: _t->logCopyComplete(); break;
        case 22: _t->on_mListLog_customContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)()>(_a, &ResultsView::gotResults, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)(bool )>(_a, &ResultsView::resultsHidden, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)(QStringList )>(_a, &ResultsView::checkSelected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)(QStringList )>(_a, &ResultsView::suppressIds, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)(ShowTypes::ShowType , bool )>(_a, &ResultsView::showResults, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)(bool )>(_a, &ResultsView::showCppcheckResults, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)(bool )>(_a, &ResultsView::showClangResults, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)()>(_a, &ResultsView::collapseAllResults, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)()>(_a, &ResultsView::expandAllResults, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ResultsView::*)()>(_a, &ResultsView::showHiddenResults, 9))
            return;
    }
}

const QMetaObject *ResultsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResultsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ResultsViewE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ResultsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void ResultsView::gotResults()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ResultsView::resultsHidden(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ResultsView::checkSelected(QStringList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ResultsView::suppressIds(QStringList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ResultsView::showResults(ShowTypes::ShowType _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void ResultsView::showCppcheckResults(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ResultsView::showClangResults(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ResultsView::collapseAllResults()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ResultsView::expandAllResults()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ResultsView::showHiddenResults()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
