/****************************************************************************
** Meta object code from reading C++ file 'threadhandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../gui/threadhandler.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'threadhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ThreadHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto ThreadHandler::qt_create_metaobjectdata<qt_meta_tag_ZN13ThreadHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ThreadHandler",
        "done",
        "",
        "log",
        "msg",
        "debugError",
        "ErrorItem",
        "item",
        "threadDetailsUpdated",
        "QMap<int,CheckThread::Details>",
        "threadDetails",
        "progress",
        "filename",
        "stage",
        "std::size_t",
        "value",
        "stop",
        "startCheck",
        "CheckThread::Details",
        "details",
        "finishCheck",
        "threadDone"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'done'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'log'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'debugError'
        QtMocHelpers::SignalData<void(const ErrorItem &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'threadDetailsUpdated'
        QtMocHelpers::SignalData<void(QMap<int,CheckThread::Details>)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(QString, QString, std::size_t)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startCheck'
        QtMocHelpers::SlotData<void(CheckThread::Details)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'finishCheck'
        QtMocHelpers::SlotData<void(CheckThread::Details)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'threadDone'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ThreadHandler, qt_meta_tag_ZN13ThreadHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ThreadHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ThreadHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ThreadHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ThreadHandlerE_t>.metaTypes,
    nullptr
} };

void ThreadHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ThreadHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->done(); break;
        case 1: _t->log((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->debugError((*reinterpret_cast<std::add_pointer_t<ErrorItem>>(_a[1]))); break;
        case 3: _t->threadDetailsUpdated((*reinterpret_cast<std::add_pointer_t<QMap<int,CheckThread::Details>>>(_a[1]))); break;
        case 4: _t->progress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::size_t>>(_a[3]))); break;
        case 5: _t->stop(); break;
        case 6: _t->startCheck((*reinterpret_cast<std::add_pointer_t<CheckThread::Details>>(_a[1]))); break;
        case 7: _t->finishCheck((*reinterpret_cast<std::add_pointer_t<CheckThread::Details>>(_a[1]))); break;
        case 8: _t->threadDone(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CheckThread::Details >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CheckThread::Details >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ThreadHandler::*)()>(_a, &ThreadHandler::done, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThreadHandler::*)(const QString & )>(_a, &ThreadHandler::log, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThreadHandler::*)(const ErrorItem & )>(_a, &ThreadHandler::debugError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThreadHandler::*)(QMap<int,CheckThread::Details> )>(_a, &ThreadHandler::threadDetailsUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ThreadHandler::*)(QString , QString , std::size_t )>(_a, &ThreadHandler::progress, 4))
            return;
    }
}

const QMetaObject *ThreadHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ThreadHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ThreadHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ThreadHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void ThreadHandler::done()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ThreadHandler::log(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ThreadHandler::debugError(const ErrorItem & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ThreadHandler::threadDetailsUpdated(QMap<int,CheckThread::Details> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ThreadHandler::progress(QString _t1, QString _t2, std::size_t _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
